#===============================================================================================
# FUNCTION for generating list of intervals 
# and components from Peak list
# 
# fin -> file with a peak list -> string
#    It assumes that 1st and 2nd column are No of peak and 
#    HN peak position (x0) in points, respectively
# col -> No of column in the list with HN (ppm) -> integer 
# colN -> No of column in the list with N (ppm) -> integer
# colI -> No of column in the list with Peak Intensity -> integer
# m -> number of points for an interval (half range)-> float
# fac -> shifts in an interval of x0+/-fac*m will be taken -> float
# peakf-> parameter telling which PeakFunc(type of function: Gaussian or Lorentzian) to choose
# n -> half-width at half-maximum (HWHM) in points -> float
# tresh -> treshold for the ratio: Lorentzian(x)/Intensity(x0) -> float 
#    such that a neighbour peak would give a component for x0
#=============================================================================================

def peak2comp(fin, col, colN, colI, m, mpar, peakf, n, tresh):
	#**********************************************************
	# Lorentzian/Gaussian function
	# x0: peak position; Lx0: Intensity at x0; 
	# hwhm: half-width at half-maximum; x: position of interest
	#**********************************************************
	col, colN, colI = int(col), int(colN), int(colI)
	if peakf == 1:
		PeakFunc='G'
	elif peakf == 2:
		PeakFunc='L'
	else:
		PeakFunc=[]
		print "\n!ERROR: Undefined peak function\n"
		

	def LorF(Lx0,x0,hwhm,x):	
		return Lx0/(1+((x-x0)/hwhm)**2)
	
	def GausF(Lx0,x0,hwhm,x):
		return Lx0* exp(-log(2)*((x-x0)/hwhm)**2)
	
	# Reading the data
	f = open(fin)
	buf = f.readlines()
	f.close()
	
	# Preparing the output
	IntlRes = ['NoPeak'+' '+'StartIntl'+' '+'EndIntl'+' '+'NoComp'+' '+'HN-shift'+' '+'N-shift'+'\n']
	# Pini: HN peak position in points; IPini: Peak Intensity 
	Pini, IPini = [], []
	# HN, N - positions in ppm; pN - No of the peak
	csHN, csN, pN = [],[],[]
	#Finding the first line with peaks
	for i in range(len(buf)):
    		if len(buf[i].split())!=0:
        		par = ord(buf[i].split()[0][0])
        		#looks only into lines starting with number
        		if logical_and(par >= 48, par <= 57):
	    			pN.append(int(buf[i].split()[0]))
            			Pini.append(float(buf[i].split()[1])-1)
	    			IPini.append(float(buf[i].split()[colI]))    
	    			csHN.append(float(buf[i].split()[col]))
	    			csN.append(float(buf[i].split()[colN]))	    
    
	#Sortinf the peak list 
	lind = array(csHN).argsort().tolist()
	# check for repeating indices
	ind = [x for x in lind if lind.count(x)>1]

	if len(ind):
		print "!Warning: there is a problem and the list will not be sorted"
		Plst,IPlst,csHNlst,csNlst,pNlst = Pini,IPini,csHN,csN,pN
	else:
		Plst = [Pini[x] for x in lind]
		IPlst = [IPini[x] for x in lind]
		csHNlst = [csHN[x] for x in lind]
		csNlst = [csN[x] for x in lind]
		pNlst = [pN[x] for x in lind]
	
	stI = [int(round(x-m)) for x in Plst]
	endI = [int(round(x+m)) for x in Plst]

	# Shifts in a range of +/-dmin around x0 will be considered 
	dmin = mpar*m
	k = range(len(Plst))
	comp = []
	#counts how many components are found in an interval using intensities of neighbouring resonances
	for j in range(len(Plst)):   
    		indcomp = [x for x in k if logical_and(abs(Plst[x]-Plst[j])<=dmin,x!=j)]
		Prest = [Plst[x] for x in indcomp]
		IPrest = [IPlst[x] for x in indcomp]	
    		if len(Prest):
			# split the neighbours in 2: right and left from the j-one 
			RPrest= [Prest[x] for x in range(len(Prest)) if Prest[x] > Plst[j]]
			IRPrest= [IPrest[x] for x in range(len(Prest)) if Prest[x] > Plst[j]]
			LPrest= [Prest[x] for x in range(len(Prest)) if Prest[x] < Plst[j]]
			ILPrest= [IPrest[x] for x in range(len(Prest)) if Prest[x] < Plst[j]]
		
			Rcomp, Lcomp = 0, 0
			if len(RPrest):
				if PeakFunc=='G':
					Icalc = [GausF(IRPrest[x],RPrest[x],n,endI[j]) for x in range(len(RPrest))]
				elif PeakFunc=='L':
					Icalc = [LorF(IRPrest[x],RPrest[x],n,endI[j]) for x in range(len(RPrest))]
				res = [x for x in Icalc if x > tresh*IPlst[j]]
 				Rcomp = len(res)
				
			if len(LPrest):
				if PeakFunc=='G':
					Icalc = [GausF(ILPrest[x],LPrest[x],n,stI[j]) for x in range(len(LPrest))]
				elif PeakFunc=='L':
					Icalc = [LorF(ILPrest[x],LPrest[x],n,stI[j]) for x in range(len(LPrest))]
				res = [x for x in Icalc if x > tresh*IPlst[j]]
 				Lcomp = len(res)
			comp.append(Rcomp+Lcomp+1)			
    		else:
			# no neighbouring resonances in a range
			comp.append(1)

	[IntlRes.append(str(pNlst[x])+' \t'+str(stI[x])+' \t'+str(endI[x])+' \t'+str(comp[x])+' \t'+str(csHNlst[x])+' \t'+str(csNlst[x])+'\n') \
		for x in range(len(pNlst))]
	
	return IntlRes 
#======
# MAIN
#======
from numpy import *
import tkFileDialog
import pdb

if __name__ == "__main__":
	# for test	
	result = peak2comp('peak2.tab', 5, 6, 17, 2.5, 10., 'G', 2., 0.1)
	f = open('ftest','w')
	f.writelines(result)
	f.close()


