from prodecomp import *
from numpy import loadtxt
import struct, os

#==Function for INPUT and OUTPUT of prodecomp
# DefsFile -> file with definitions (WB format)
# FileList -> selection of data to be used in a decomposition
# Intl, NumComp, RegFac, NumIter -> interval, No of components, regular. factor and No of iterations
def IOprodec(DefsFile, FileList, Intl, NumComp, RegFac, NumIter):
    f = open(DefsFile,'r')
    buf = f.readlines()
    f.close()
    readindic = 0	# indicator for reading ascii data
    for row in buf:
	if row.find('SIZE=')!=-1:
		xsize = int(row.split('(')[1].split(')')[0])
		ysize = int(row.split(',')[1].strip())
	elif row.find('FORMAT=')!=-1:
		# open 2rr (Bruker) data: 4-bytes integer values in binary format, no header 
		if row.split()[-1] == 'Bruker':	btype, rmode, header  = 'i', 'rb', 0
		# NMRPipe data: 4-bytes floating-point values
		elif row.split()[-1] == 'FT2': btype, rmode, header  = 'f', 'rb', 512
		elif row.split()[-1] == 'CEP': btype, rmode, header  = 'f', 'rb', 0
		elif row.split()[-1] == 'Ascii': readindic = 1
		

    RemoveRow=[]
    for i, elem in enumerate(buf):
		if elem.find('PATH:')!=-1:
			if all([elem.find(x)==-1 for x in FileList]):				
				RemoveRow.append(i)
				RemoveRow.append(i+1)
    bufin = [buf[x] for x in range(len(buf)) if x not in RemoveRow] 
    
    Ldata = []
    Ldefs = []
    
    for num in range(len(bufin)):
            if bufin[num].find('NUCLEI')!=-1:
                    # tpfdir and typedefs -> type of nuclei for direct and indirect dimensions resp.
                    Lnuclei = [x.split()[0] for x in bufin[num].split('=')[1].split(',')]
                    typedefs = Lnuclei[1::]
                    tpfdir = [Lnuclei[0]]           
            elif bufin[num].find('PATH')!=-1:                        
                    floc = bufin[num].split('PATH:')[1].split('\\\n')[0].strip()
                    # Ascii data
                    if readindic: 
                        data = loadtxt(floc)
                        if data.shape!=(ysize,xsize): print "!Warning: The SIZE parameters do not match with the data size."				
                    # Binary data
                    else:
                        # the path is a Bruker directory
                        if not len(floc.split(os.sep)[-1]): f = open(floc+'2rr',rmode)
                        # the path is a full name (directory+file)		    
                        else: f = open(floc,rmode)
                        buf = f.read()
                        f.close()
                        FullData = array(struct.unpack(btype*(len(buf)/struct.calcsize(btype)),buf))
                        data = FullData[header::]                  
                        data = data.reshape(ysize,xsize)

                    # Generating list of data
                    Ldata.append(data) 
                    # Assuming:
                    # -the first value in definitions is always '1' and refers to direct dim
                    # -every PATH line is followed by a DEFINITION line
                    Ldefs.append([x.split()[0] for x in bufin[num+1].split(':')[1].split(',')[1::]])

                    buf = []
                    
    # GENERATE 3D matrix Pmx by stacking the data
    # GENERATE matrix of Definitions: Nrows = Number of files; Ncolumns = Number of nuclei types    
    Pmx = zeros((ysize,xsize,len(Ldata)))
    indefs = zeros((len(Ldefs),len(Ldefs[0])))
    for i in range (len(Ldata)):
            Pmx[:,:,i]=Ldata[i]
            indefs[i,:]=array([int(x) for x in Ldefs[i]])

    # REMOVE from DEFINITIONS the shapes that can not be resolved from the selection of input data
    # defs-> selection from indefs; tpf-> corresponding selection from typedefs    
    clist = []
    defs = zeros((len(Ldata),1))
    for j in range(len(typedefs)):
        # select columns(shapes) that have at least one non-zero element
        if any(indefs[:,j])!=0:
           clist.append(j) 
            
    defs = indefs[:,clist]
    tpf = []
    for k in clist:
        tpf.append(typedefs[k])
    
    #==PRODECOMP Arguments==
    # Pmx and defs -> matrices of data and corresponding definitions, resp.
    # Intl -> decomposition interval along direct dim.
    # NumComp -> number of components
    # RegFac, NumIter -> regularization factor and number of iterations, resp. 
    DecRes = prodecomp(Pmx, defs,Intl, [NumComp,NumComp], RegFac, NumIter)
 
    return  DecRes,tpfdir,tpf
	
if __name__ == "__main__":
	import pdb

	DefsFile = './testdata/prodecomp.txt'
	FileList = [' /home/dory/WORK/PROGRAMS/Development/SHABBA/prodecomp/testdata/s101.ft2 \\',' /home/dory/WORK/PROGRAMS/Development/SHABBA/prodecomp/testdata/s112.ft2 \\']	
	#IntlStart, IntlEnd, NumComp, RegFac, NumIter= 459, 465, 2, 0.1, 3
        IntlStart, IntlEnd, NumComp, RegFac, NumIter= 76, 86, 2, 0.1, 3
	Intl = [IntlStart, IntlEnd]
	[[outfdir, outf], tpfdir, tpf]=IOprodec(DefsFile, FileList, Intl, NumComp, RegFac, NumIter)
	pdb.set_trace()

