"""
======================COPYRIGHT/LICENSE START==========================

guiPRODv3.py: Graphycal User Interface for PRODECOMP program

Copyright (C) 2008 Doroteya K. Staykova, Daniel Malmodin, Martin Billeter 
(University of Gothenburg, Sweden)

=======================================================================

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
======================COPYRIGHT/LICENSE END============================

for further information, please contact :
- our website (http://www2.chem.gu.se/bcbp/nmr/)
- dkstaykova@gmail.com, martin.billeter@chem.gu.se 

=======================================================================

If you are using this software for academic purposes, we suggest
quoting (one of) the following references:

===========================REFERENCE START=============================
Malmodin, D. and Billeter, M. (2005) Multiway Decomposition of NMR Spectra 
with Coupled Evolution Periods. J. Am. Chem. Soc., 127, 13486-13487. 

Malmodin, D. and Billeter, M. (2006) Robust and versatile interpretation 
of spectra with coupled evolution periods using multi-way decomposition. 
Magn. Reson. Chem., 44, S185-S195.

Staykova, D. K.; Fredriksson, J.; Bermel, W. and Billeter, M. (2008) 
Assignment of protein NMR spectra based on projections, multi-way decomposition 
and a fast correlation approach. J. Biomol. NMR., 42, 87-97.

===========================REFERENCE END===============================
"""

class guiPRODECOMP(object):
        def __init__(self, master=None):
                self.master = master
                self.value = None                 
		self.dirout, self.header = None, None
		
                self.TL = Toplevel(master)
                self.TL.title("PRODECOMP (PROjection DECOMPosition)")               
             
		print docprodec.__doc__  
                #***************
                # DATA selection
                #***************
                #--------frameF0--------------
                F0 = Frame(self.TL, padx = 50)                
                F0.pack(side=TOP)                
 
                self.M = Button(F0, text = "Load Definition file", bg = "yellow", relief = RAISED, command = self.readdata)
                self.M.pack(side=TOP)
                
		self.NucType = Label(F0, text=' ')
		self.NucType.pack(side=TOP)
               
                self.s = Scrollbar(F0, orient=VERTICAL)
		self.sh = Scrollbar(F0, orient=HORIZONTAL)
                self.L = Listbox(F0, width=int(gcf().get_size_inches()[0])*10, height=int(gcf().get_size_inches()[0])*2,\
			bg='white', selectmode=EXTENDED, yscrollcommand=self.s.set, xscrollcommand=self.sh.set)
                self.s.config(command=self.L.yview)
		self.sh.config(command=self.L.xview)
                self.s.pack(side=RIGHT, fill=Y)
		self.sh.pack(side=BOTTOM, fill=X)
                self.L.pack(side=LEFT)                 
                
                #--------frameF1--------------
                F1 = Frame(self.TL)
                F1.pack(side=TOP)
                self.chooseButton = Button(F1, bg = "black",fg = "white", width = 15, text="Take Selection", command=self.choice)
                self.chooseButton.pack(side=LEFT, padx = 8)
		self.chooseButton.config(state=DISABLED)

                self.clearButton = Button(F1, bg = "white",fg = "black", width = 15, text="Change Selection", command=self.chngsel)
                self.clearButton.pack(side=LEFT, padx = 8)
		self.clearButton.config(state=DISABLED)

                #*****************************
                # PARAMETERS and RUN PRODECOMP
                #*****************************		
		self.prodButton = Button(self.TL, bg = "yellow", width = 37, text = "INPUT -> PRODECOMP -> OUTPUT", command = self.setparam)
                self.prodButton.config(state=DISABLED)
                self.prodButton.pack(side=TOP)               
                    
        def readdata(self, event=None):                                                      
                FileDefs = askopenfilename(title='Select a definition file')
		try:
			f = open(FileDefs,'r')
			buf = f.readlines()
			self.file4read = FileDefs
			f.close()
			self.M.configure(state=DISABLED)
			for num in range(len(buf)):
            			if buf[num].find('NUCLEI')!=-1:	
					self.NucType.config(text=buf[num].strip())
					self.NucType.update_idletasks()
				elif buf[num].find('PATH')!=-1: self.L.insert(END, buf[num+1].strip()+ 2*' '+ buf[num].split('=')[-1].strip())
				elif buf[num].find('SIZE')!=-1: self.DirSize = int(buf[num].split('(')[1].split(')')[0])
			self.chooseButton.config(state=NORMAL)
			self.clearButton.config(state=NORMAL)

		except: tkMessageBox.showerror(title=None, message="Problem with file opening.")		 
               
        def choice(self,event=None):                
                try:
                        Index = self.L.curselection()                        
                        self.value = map(int,Index)                                         
                        self.fselect = [self.L.get(x).split(':')[-1] for x in self.value]
                except IndexError:
                        self.value = None                
                
                self.L.config(state=DISABLED)
		self.prodButton.config(state=NORMAL)

        def chngsel(self, event=None):
                self.L.config(state=NORMAL)                
	
	# Parameters for Prodecomp
	def setparam(self,event=None):
		self.ParamTL = Toplevel(self.TL)
                self.ParamTL.title("INPUT -> PRODECOMP -> OUTPUT")
		
		if tkMessageBox.askokcancel(title='Initialization', message='Do you want to open FILE with parameters?'):
            		f = open(askopenfilename(title="File with intervals/components"),'r')
			buf = f.readlines()			
        	else:
            		buf = ['0 \t-1 \t-1 \t-1 \t-1 \t-1\n']
		
		
		frmNitNr = Frame(self.ParamTL)
		frmNitNr.grid(row=0, column=0, columnspan=5)
		Label(frmNitNr, width=12,text="Regularisation \nfactor").grid(row=0, column=0)
		Label(frmNitNr, width=12,text="Number of \niterations").grid(row=0, column=3)
		Label(frmNitNr, width=12).grid(row=0, column=2)
		
		self.INregfac = StringVar()                
                self.regfac = Entry(frmNitNr, bg='white', width=5, relief=RIDGE, textvariable = self.INregfac)
		self.regfac.grid(row=0, column=1)
                self.regfac.insert(END,'0.1')

		self.INniter = StringVar()                
                self.niter = Entry(frmNitNr, bg='white', width=5, relief=RIDGE, textvariable = self.INniter)
		self.niter.grid(row=0, column=4)
                self.niter.insert(END,'3') 
              
		# 2+4 parameters: reg. fac; Niterations; start (end) interval; Ncomponents; component of interest
		InputPar = ['\nInterval No\n', '\nStart,\npoints ','\nEnd,\npoints ',\
			    '\nNumber of\ncomponents', '\nSelect, \ncomp No']		
		frmIP = Frame(self.ParamTL)
		frmIP.grid(row=1, column=0, columnspan=5)
		for i in range(len(InputPar)): Label(frmIP, width=10, text=InputPar[i]).grid(row=2, column=i)

		F0=Frame(self.ParamTL,relief=RIDGE, bd=2)
		F0.grid(row=2, column=0, columnspan=6)			
		self.cnv = Canvas(F0, height=int(gcf().get_size_inches()[0])*int(gcf().get_size_inches()[1])**2)
		self.cnv.grid(row=0, column=0)			
		hScroll = Scrollbar(F0, orient=HORIZONTAL, command=self.cnv.xview)
		hScroll.grid(row=1, column=0,sticky='we')
		vScroll = Scrollbar(F0, orient=VERTICAL, command=self.cnv.yview)
		vScroll.grid(row=0, column=2, sticky='ns')
		self.cnv.configure(xscrollcommand=hScroll.set, yscrollcommand=vScroll.set)
		self.F1 = Frame(self.cnv)
		self.cnv.create_window(0, 0, window=self.F1, anchor='nw')		
	
		pStInt, pEnInt, pNumComp, pComp = [],[],[],[]
		for i in range(len(buf)):
			if len(buf[i].split())!=0:
        			par = ord(buf[i].split()[0][0])
        			#looks only into lines starting with number
        			if logical_and(par >= 48, par <= 57):
					if buf[i].split()[1]=='PATH:':
						var = buf[i].split()[2].split(os.sep)[-2]
						pStInt.append(int(var.split('_')[0]))
						pEnInt.append(int(var.split('_')[1].split('c')[0]))
						pNumComp.append(int(var.split('_')[1].split('c')[1]))
						pComp.append(int(buf[i+1].split()[1]))
					else:			
	    					pStInt.append(int(buf[i].split()[1]))
						pEnInt.append(int(buf[i].split()[2]))
						pNumComp.append(int(buf[i].split()[3]))	
					
		self.InPar = zeros((len(pStInt),5))
		self.InPar[:,0] = range(len(pStInt))
		self.InPar[:,1], self.InPar[:,2], self.InPar[:,3]= pStInt, pEnInt, pNumComp
		if len(pComp): self.InPar[:,4] = pComp
		else: self.InPar[:,4] = -1
		self.param = [[] for x in range (self.InPar.shape[1])]
		self.FillCanvas(self.InPar)
		
		frmBut = Frame(F0)
		frmBut.grid(row=2, column=0)
		rind = 0
		WidthSize = 20
		Button(frmBut,text='Run ONE', bg = 'light yellow', width=WidthSize, command=self.runOne).grid(row=rind, column=0)
		Button(frmBut,text='Run/Save ALL', bg = 'dark grey', command=self.runMany).grid(row=rind+1, column=1,sticky='nswe')
		self.plotBut = Button(frmBut,text='Plot ONE', bg = "light grey", disabledforeground = 'white', width=WidthSize, command = self.PlotOne)
		self.plotBut.grid(row=rind, column=1)
		self.plotBut.config(state=DISABLED)
		self.saveBut = Button(frmBut,text='Save CURRENT', bg = 'yellow', disabledforeground = 'white', command = self.saveout)
		self.saveBut.grid(row=rind+1, column=0, sticky='nswe')
		self.saveBut.config(state=DISABLED)
		Button(frmBut,text='Add ONE', bg = 'white', command=self.AddInt).grid(row=rind+2, column=0, sticky='nswe')
		Button(frmBut,text='Remove ONE', bg = 'black', fg = 'white', command=self.RemInt).grid(row=rind+2, column=1, sticky='nswe')
		self.CorrBut = Button(frmBut,text='Correlate TWO', bg = 'grey', command=self.Correl2Intl)
		self.CorrBut.config(state=DISABLED)
		self.CorrBut.grid(row=rind+3, column=0, sticky='nswe')
		Button(frmBut,text='Save LIST with ALL', bg = 'yellow', command=self.SaveList).grid(row=rind+3, column=1, sticky='nswe')		
	
	def FillCanvas(self,inputparam):
                for i in range(inputparam.shape[1]):
                    for j in range(inputparam.shape[0]):
			if i:                          
                        	e = Entry(self.F1, width=9, relief=RIDGE, bg='white')
                        	e.grid(row=j, column=i)
                        	e.insert(END, int(inputparam[j, i]))
                        	
			else:	
				e = Entry(self.F1, width=13, relief=FLAT, disabledforeground="black")
				e.grid(row=j, column=i)
				e.insert(END, int(inputparam[j, i]))				
				e.config(state=DISABLED)
			self.param[i].append(e)
		
		self.F1.update_idletasks()		
		#self.cnv.configure(scrollregion=(0, 0, self.F1.winfo_width(), self.F1.winfo_height()))	

	def Correl2Intl(self, event = None):
		def CalcCorr():
			# Reading intervals and making tree structure: Intervals(i)->Components(c)->Shapes(s)
			try:
				IntSel = [int(x) for x in [self.Int1No.get(), self.Int2No.get()]]
				ActiveRes = [[x[i] for x in self.param] for i in IntSel]			
				Intl4Corr = [self.dirout+ x[1].get()+ '_' + x[2].get() + 'c' + x[3].get() + os.sep for x in ActiveRes]
				Lintl = [[] for i in range(len(Intl4Corr))]
                                for i in range(len(Intl4Corr)):
                                        if os.path.exists(Intl4Corr[i]):
                                                Ncomp = int(Intl4Corr[i].split(os.sep)[-2].split('c')[-1])	
                                                slist = []
                                                for jf in glob.glob(Intl4Corr[i] +'f*'):
                                                        slist.append(jf.split(Intl4Corr[i])[-1])
                                                # range of indirect shapes: from 1 to max
                                                Nshp = max([int(x.split('s')[1].split('c')[0]) for x in slist if 'fdir' not in x])
                                                # Tree structure
                                                Lintl[i]= [[[] for s in range(Nshp)] for k in range(Ncomp)]
                                                for c in range(Ncomp):
                                                        for s in range(Nshp):
                                                                patch = 's'+str(s+1)+'c'+str(c)
                                                                for elem in slist:
                                                                        if patch in elem:
                                                                                f = open(Intl4Corr[i]+elem,'r')
                                                                                buf = f.readlines()
                                                                                f.close()
                                                                                for m in range(len(buf)):
                                                                                        Lintl[i][c][s].append(float(buf[m].split()[0]))
					else: 
						tkMessageBox.showerror(title=None, message="Selected intervals don't exist!")
						break
				
                                # Calculating correlation of components	for 2 intervals (all shapes for a component -> vector with length = SizeShape*NShapes)
                                CorelatTab = zeros((len(Lintl[1]),len(Lintl[0])))
                                for j in range(len(Lintl[0])):
                                        Rvec = []
                                        for k in range(Nshp): Rvec = Rvec+Lintl[0][j][k]
                                        for i in range(len(Lintl[1])):
                                                Cvec = []		
                                                for k in range(Nshp): Cvec = Cvec+Lintl[1][i][k]
                                                CorelatTab[i, j] = round(corrcoef(Cvec, Rvec)[1,0]*100,1)
                                
                                RowName = ['comp '+str(i) for i in range(CorelatTab.shape[0])]
                                ColName = ['comp '+str(i) for i in range(CorelatTab.shape[1])]
                                # Create a table (widget) with the information (columnwise)
				try:
					self.FrmTab.destroy()
				except:
					pass
				self.FrmTab = Frame(self.CorrRes , relief=RIDGE, bd=2)
				self.FrmTab.grid(row=1,column=1, columnspan=len(ColName)+1)
                                TabText = [[] for x in range(len(ColName)+ 1)]
                                TabText[0] = [' ']+RowName
				[Label(self.FrmTab, width = 10, text = TabText[0][i],bg='white', relief=RIDGE).grid(row=i, column=0) for i in range(len(TabText[0]))]			
                                for j in range(len(ColName)):
                                        TabText[j+1] = [ColName[j]] + [str(x) for x in CorelatTab[:,j]]
					for i in range(len(TabText[j+1])):
						try:
							CheckVal = float(TabText[j+1][i])
							if CheckVal>50.: bgvalue = 'yellow'
							else: bgvalue = 'white'
						except:
							bgvalue = 'white'						
						Label(self.FrmTab, width = 10, text = TabText[j+1][i], bg=bgvalue,relief=RIDGE).grid(row=i, column=j+1)						
				
			except:	
				tkMessageBox.showerror(title=None, message="Problem reading intervals, \nplease check interval No!")	
				
		
		# Function entry->Export correlation results for 2 intervals in a table
		if self.dirout==None: tkMessageBox.showerror(title=None, message="Please save a result with at least one decomposition!") 
		else:
			self.CorrRes = Toplevel(self.TL) 	
			self.CorrRes.title("Correlation of decompositions")
			frmI1 = Frame(self.CorrRes)
			frmI1.grid(row=0, column=1)	              
			Label(frmI1, text="Interval B:").grid(row=0, column=0) 
			self.Int1No = StringVar()                
                	Entry(frmI1, width=8, bg = 'dark gray', relief=RIDGE, textvariable = self.Int1No).grid(row=0, column=1)
			frmI2 = Frame(self.CorrRes)
			frmI2.grid(row=1, column=0)
			Label(frmI2, text="Interval A:").grid(row=0, column=0, sticky='we')
 			self.Int2No = StringVar()
			Entry(frmI2, width=8, bg = 'dark gray', relief=RIDGE, textvariable = self.Int2No).grid(row=0, column=1)	
			Button(self.CorrRes,text='Show result', command=CalcCorr).grid(row=0, column=2)
			
	def SaveList(self, event = None):
		#==Function for saving decomposition results in a fashion similar to definition file
		def SaveComp(OutFile, DirComp, Header, pvalue):
			f = open(OutFile,'w')	
			f.write(Header)
			f.close()
			output = []
			for i, elem in enumerate(pvalue):
				if not i: # number of output component 
					for k in elem: output.append(k+'= ')
				elif i==1: # start interval
					for j, k in enumerate(elem): output[j] = output[j]+'PATH: '+DirComp+k
				elif i==2: # end interval
					for j, k in enumerate(elem): output[j] = output[j]+'_'+k
				elif i==3: # number of components used
					for j, k in enumerate(elem): output[j] = output[j]+'c'+k +os.sep+' '+'\\\n'
				elif i==4: # component to select
					for j, k in enumerate(elem): output[j] = output[j]+4*' '+'COMPONENT: '+k+'\n'

			f = open(OutFile,'a')
			[f.write(x) for x in output]
			f.close()
			return
		#self.param: list of columns with entry widgets; each column refers to one parameter				
		pvalue = [[] for i in range(len(self.param))]
                for i in range(len(self.param)):			                   
                        for col in self.param[i]:
                                pvalue[i].append(col.get(),)		
		
		if self.dirout==None: tkMessageBox.showerror(title=None, message="Please save a result with at least one decomposition!") 
		elif self.header==None: tkMessageBox.showerror(title=None, message="Please plot at least one result!")
		else: 
			fileout = asksaveasfilename(title='File to save the results')
			SaveComp(fileout, self.dirout, self.header, pvalue)

	def AddInt(self, event = None):
		var = max([int(x.get()) for x in self.param[0]])
		if var:
			self.OpenWin(self.insertOne)
		else:
			newpar = zeros((1,len(self.param)))
			newpar[0] = var+1
			newpar[0,1::]=-1						
			self.FillCanvas(newpar)

	def insertOne(self, event = None):
		IntSel = int(self.IntNo.get())
		self.ChoiceIntl.destroy()
		for i in range(len(self.param)):
			f = Frame(self.F1)
                	f.pack(side=TOP)
			e = Entry(f)
			if i:				
				e.insert(END, int(self.param[i][IntSel].get()))
			else:
				e.insert(END, IntSel+1)
			self.param[i].insert(IntSel+1,e)
		self.updateCanvas()

	def RemInt(self, event = None):
		self.OpenWin(self.removeOne)

	def removeOne(self, event = None):
		IntSel = int(self.IntNo.get())
		self.ChoiceIntl.destroy()
 		[x[IntSel].config(state=NORMAL)for x in self.param]
		[x[IntSel].delete(0, END) for x in self.param]
		[x.pop(IntSel) for x in self.param]
		self.updateCanvas()

	def updateCanvas(self, event = None):			
		pvalue = [[] for i in range(len(self.param))]
                for i in range(len(self.param)):			                   
                        for col in self.param[i]:				
                                pvalue[i].append(int(col.get()),)
		
		pvalue[0] = range(len(pvalue[0]))
		self.param = [[] for x in range (len(pvalue))]
		self.F1.destroy()
		self.F1 = Frame(self.cnv)
		self.cnv.create_window(0, 0, window=self.F1, anchor='nw')
	 	self.FillCanvas(transpose((array(pvalue))))		
	
	# Decomposition of interval No (IntSel)
        def runprodecomp(self,IntSel): 
		runparam = [int(x[IntSel].get()) for x in self.param]
		self.interval = [runparam[1], runparam[2]]
                self.ncomps = runparam[3] 
		self.regular = float(self.INregfac.get())
                self.iterations = int(self.INniter.get())-1
		print "\nRunning interval No", IntSel,"..."
		try: 
                	[[self.outfdir, self.outf], self.tpfdir, self.tpf] = IOprodec(self.file4read, self.fselect, self.interval, self.ncomps, self.regular, self.iterations) 
                	print "Decomposition is done!"

		except:	tkMessageBox.showerror(title=None, message="!ERROR in the input for prodecomp. PLEASE CHECK the definitions or the data.")
			

	# Opens a new window for selecting an interval and executing choosen command (com)
	def OpenWin(self,com):
		self.ChoiceIntl = Toplevel(self.TL)		              
		Label(self.ChoiceIntl, text="Type the interval No:", bg = "yellow").pack(side=TOP) 
		self.IntNo = StringVar()                
                IntEnt = Entry(self.ChoiceIntl, width=8, relief=RIDGE, textvariable = self.IntNo)
		IntEnt.pack(side=LEFT)
		Button(self.ChoiceIntl,text='Select', command=com).pack(side=LEFT)

	def RunDec(self, event=None):
		self.ChoiceIntl.destroy()
		self.runprodecomp(int(self.IntNo.get()))

	def runOne(self,event=None):						
		self.OpenWin(self.RunDec)
		self.plotBut.config(state=NORMAL)
		self.saveBut.config(state=NORMAL)
		self.CorrBut.config(state=NORMAL)
	
	def runMany(self,event=None): 
		for i in range(len(self.param[0])):
			self.runprodecomp(i)		
			self.saveout()
		print "\nFinished!"
		self.plotBut.config(state=NORMAL) 
	
	# SAVE results 
        def saveout(self, event = None): 
		# Prodecomp OUTPUT directory-> fixed
                self.dirout = os.getcwd()+ os.sep+'ProdecompOutput' + os.sep
		if not os.path.exists(self.dirout): os.mkdir(self.dirout)
                       
		Ncomp = self.outfdir.shape[1]
                # ASCII OUTPUT of PRODECOMP
                decout = self.dirout + str(self.interval[0])+ '_' + str(self.interval[1]) + 'c' + str(Ncomp) + os.sep
                if os.path.exists(decout): shutil.rmtree(decout)
                os.mkdir(decout)

                # s1 -size of fdir; s2 - number of components
                [s1, s2] = self.outfdir.shape
                for i in range(s2):                        
                        fname = decout + 'fdir' + str(self.interval[0])+ '_' + str(self.interval[1]) +'c'+ str(i)                        
			savetxt(fname,transpose((self.outfdir[:,i]))) 

                # d1 -size along the indir dim; d2 - number of shapes; d3 - number of components
                [d1, d2, d3] = self.outf.shape
                for i in range(0,d2):
                        for j in range(0,d3):
                            fname = decout + 'f' + str(self.interval[0])+ '_' + str(self.interval[1]) +'s'+ str(i+1)+'c'+ str(j)
			    savetxt(fname,transpose((self.outf[:,i,j])))                           

                print 'ASCII->'+ decout
                
        # PLOT results
	def PlotOne(self,event=None):
		if self.dirout==None: tkMessageBox.showerror(title=None, message="Please save a result with at least one decomposition!") 			
		else: self.OpenWin(self.PlotInt)

        def PlotInt(self, event=None):			
		self.ChoiceIntl.destroy() 		
		self.plotshapes(int(self.IntNo.get()))

        def plotshapes(self, IntSel):
		# LOAD selected results from ASCII directory
		# !Warning: assuming nuclei types (direct/indirect dimension) for all intervals in the active window are the same 
		ActiveRes = [x[IntSel] for x in self.param]		
		self.interval = [int(ActiveRes[1].get()), int(ActiveRes[2].get())]
                comps = ActiveRes[3].get()
		
                DatRes = self.dirout+ str(self.interval[0])+ '_' + str(self.interval[1]) + 'c' + comps + os.sep	
                if os.path.exists(DatRes):
                        slist = []
                        for jf in glob.glob(DatRes+'f*'):
                                slist.append(jf.split(DatRes)[-1])
                        
                        #splitting the shapes 
                        sfdir = []
                        cfdir = []
                        sf = []
                        nsf = []
                        ncf = []
                        for i in range(len(slist)):
                                if slist[i].find('fdir')!=-1:
                                        sfdir.append(slist[i])
                                        cfdir.append(int(slist[i].split('c')[-1]))
                                else:
                                        sf.append(slist[i])
                                        #indicate shape number
                                        nsf.append(int(slist[i].split('s')[1].split('c')[0]))
                                        #indicate component number
                                        ncf.append(int(slist[i].split('s')[1].split('c')[1]))
                                        
                        order = sort(cfdir)		
                        lind = [cfdir.index(x) for x in order]		
                        osfdir = [sfdir[x] for x in lind]

                        #determine number of shapes (indirect dim)
                        Nsh = max(nsf)
                        #determine the range of fdir
                        infd = int(osfdir[0].split('_')[0].split('fdir')[-1])
                        endfd = int(osfdir[0].split('_')[1].split('c')[0])
                        Npdir = endfd-infd+1
                        #determine number of components
                        Ncomp = len(sfdir)
                        self.outfdir = zeros((Npdir,Ncomp))
                        for j in range(len(osfdir)):
                               f = open(DatRes+osfdir[j],'r')
                               buf = f.readlines()
                               f.close()
                               for k in range(len(buf)):
                                       self.outfdir[k,j] = float(buf[k].split()[0])

                        #determine f(indir.dim,Nshape,Ncomp)
                        f = open(DatRes+sf[0])
                        buf = f.readlines()
                        f.close()
                        Npindir = len(buf)
                        self.outf = zeros((Npindir,Nsh,Ncomp))
                        for m in range(len(sf)):
                                if m==0:
                                        pass
                                else:
                                        f=open(DatRes+sf[m])
                                        buf = f.readlines()
                                        f.close()
                                for n in range(Npindir):
                                        self.outf[n,nsf[m]-1,ncf[m]]=float(buf[n].split()[0])                     
                        
                        # Plotting
                        self.PlotTL = Toplevel(self.TL)
                        self.PlotTL.title("Acquisition parameters --> pt-to-ppm") 
                        #yn(ppm)= a*x(pt)+b; a: -SW(ppm)/Np; b: On/BFn + SW(ppm)/2.; OP = On/BFn
                        acquspar = ['Sweep width, ppm\n ','Transmitter frequency offset /\nMiddle of the range, ppm']
                        F0=Frame(self.PlotTL)
                        F0.pack(side=LEFT)
                        if type(self.tpfdir)==type(self.tpf):
                                LabelDim = self.tpfdir+self.tpf
                        else:
                                LabelDim = [self.tpfdir]+self.tpf
                                
                        Label(F0).pack(side=TOP)
                        Label(F0,text=' ').pack(side=TOP)
                        #matrix with acqus parameters
                        self.ParMx = zeros((len(LabelDim),len(acquspar))) 
                        # Reading parameters from definition file (WB format)
                        nuc, SW, TF = [[],[],[]]			
                        try:
                                f = open(self.file4read,'r')
                                buf = f.readlines()
                                f.close()				                      
                                for row in buf:
                                        if row.find('NUCLEI')!=-1: nuc=[x.split()[0] for x in row.split('=')[1].split(',')]
                                        elif row.find('SW_ppm')!=-1: SW = [float(x.split(')')[0].strip()) for x in row.split('(')[1].split(',')]
                                        elif row.find('O1_ppm')!=-1: TF = [float(x.split(')')[0].strip()) for x in row.split('(')[1].split(',')]
                                
                                for j in range(len(LabelDim)):
                                        Label(F0,text=LabelDim[j]).pack(side=TOP)                        
                                        match = [x for x in nuc if x==LabelDim[j]]
                                        # matching is unique
                                        if (len(match)==1):
                                                imatch = nuc.index(match[0])
                                                self.ParMx[j,0] = SW[imatch]
                                                self.ParMx[j,1] = TF[imatch]
				
				# Header for output list of components
				self.header = 'NUCLEI= '
				for x in self.tpfdir+self.tpf[:-1]: self.header=self.header+x+', '
				self.header=self.header+self.tpf[-1]+'\n'
				self.header = self.header + 'SW_ppm= ('+str(self.ParMx[0,0])+'),'
				for elem in self.ParMx[1:-1,0]: self.header = self.header+' '+str(elem)+','
				self.header = self.header+' '+str(self.ParMx[-1,0])+'\n'
				self.header = self.header+'O1_ppm= ('+str(self.ParMx[0,1])+'),'
				for elem in self.ParMx[1:-1,1]: self.header = self.header+' '+str(elem)+','
				self.header = self.header+' '+str(self.ParMx[-1,1])+'\n'
				self.header = self.header+'SIZE= ('+ str(self.DirSize)+'), '+str(self.outf.shape[0])+'\n'
				
                                F1 = Frame(self.PlotTL)
                                F1.pack(side=LEFT)
                                self.rows = []
                                for i in range(self.ParMx.shape[1]):                    
                                        f = Frame(F1)
                                        f.pack(side=LEFT)
                                        Label(f,text=acquspar[i]).pack(side=TOP)  
                                        cols = []                    
                                        for j in range(self.ParMx.shape[0]):                          
                                                e = Entry(f, bg='white', relief=RIDGE)
                                                e.pack(side=TOP, padx=5)
                                                e.insert(END, self.ParMx[j, i])
                                                cols.append(e)
                                        self.rows.append(cols)

                       		F2 = Frame(self.PlotTL)
                                F2.pack(side=RIGHT)
				Label(F2,text='Component Colours').pack(side=TOP)
				
				# Choosing the colors
				clr = ['r', 'c', 'g', 'b', 'm', 'y', 'k', 'r--', 'c--', 'g--', 'b--', 'm--', 'y--', 'k--']
				clrName = ['red (r)','cyan (c)','green (g)' ,'blue (b)', 'mangenta (m)', 'yellow (y)', 'black (k)',\
						'r--', 'c--', 'g--','b--', 'm--', 'y--', 'k--']
				self.select = []
				for i in range(self.outfdir.shape[1]):
					f = Frame(F2)
					f.pack(side=TOP)
					Label(f,text= str(i)+'->'+ clrName[i], width = 15).pack(side=LEFT)
					e = Entry(f,relief=RIDGE, bg='white',width=5)
                                        e.pack(side=RIGHT)
                                        e.insert(END, clr[i])
                                        self.select.append(e)				
                                
				Button(self.PlotTL,text='Plot ONE-BY-ONE', width = 15,bg='light yellow', command=self.PlotFmany).pack(side=BOTTOM)
 				Button(self.PlotTL,text='Plot ALL', width = 15, bg='yellow', command=self.PlotF).pack(side=BOTTOM)

                        except: tkMessageBox.showerror(title=None, message="Problem with the definition file.")

                else: tkMessageBox.showerror(title=None, message=DatRes + "doesn't exist!")        
	
	def PlotParam(self, event=None): 
		clr = []
		for elem in self.select: clr.append(elem.get())
                
                pvalue = []
                for row in self.rows:                        
                        for col in row:
                                pvalue.append(col.get(),)                        
                                
                k = 0
                for i in range(self.ParMx.shape[1]):
                        for j in range(self.ParMx.shape[0]):
                                self.ParMx[j,i]=float(pvalue[k])
                                k = k+1
                
                Ncomp = self.outfdir.shape[1]                
                if Ncomp > 14:
                        print "!Maximum 14 components can be plot in one graph"
                else:
                        # define the legend
                        lst = []
                        for ko in range(Ncomp):
                            lst = lst + ['comp' + str(ko)]
                        font= FontProperties(size='small')     
                        lnf = len(self.tpf)+1
                        xfd = range(self.interval[0],self.interval[1]+1)                        
                        xf = range(self.outf.shape[0])

		return clr, pvalue, Ncomp, lst, font, lnf, xfd, xf  
		
        def PlotF(self, event=None):  			         
                        clr, pvalue, Ncomp, lst, font, lnf, xfd, xf = self.PlotParam()
			IniSize = gcf().get_size_inches()
			figure(num=1, figsize = (IniSize[0],IniSize[1]))                        
                        # for each row (nucleus)
                        param = 9                        
                        for n in range(self.ParMx.shape[0]):
                                if lnf>param: 
                                        if n>param-1: 
                                                figure(num=2, figsize = (IniSize[0],IniSize[1]))
                                                pn = str(lnf-param)+str(1)+str(n-param+1)
                                        else: pn = str(param)+str(1)+str(n+1)
                                else: pn = str(lnf)+str(1)+str(n+1)
                                subplot(pn)
                                subplots_adjust(right=0.7)
                                if (n):
                                        if logical_and(self.ParMx[n,0]!=0., self.ParMx[n,1]!=0):
                                                ppmxf = [x*(-self.ParMx[n,0]/(self.outf.shape[0]-1))+self.ParMx[n,1]+self.ParMx[n,0]/2. for x in xf]
                                        else:
                                                ppmxf=list(xf)
                                        #indirect dimension (each shape is normalised to its maximum, except in case of 1 component)
                                        if Ncomp == 1:                        
                                                plot(ppmxf,self.outf[:,n-1,0],clr[0],linewidth=2) 
                                        else:
                                                for com in range(Ncomp):
                                                        if self.outf[:,n-1,com].max(0)==0:
                                                                fshc = self.outf[:,n-1,com].copy()
                                                        else: 
                                                                fshc = self.outf[:,n-1,com]/self.outf[:,n-1,com].max(0).copy()				
                                                        plot(ppmxf,fshc,clr[com], linewidth=2)					
                                        ylabel(self.tpf[n-1],rotation=0)
                                        xlim((max(ppmxf),min(ppmxf)))
                                        
                                else:
                                        if logical_and(self.ParMx[n,0]!=0., self.ParMx[n,1]!=0):
                                                ppmxfd = [x*(-self.ParMx[n,0]/(self.DirSize-1))+self.ParMx[n,1]+self.ParMx[n,0]/2. for x in xfd]
                                        else:
                                                ppmxfd=list(xfd)
                                        
                                        i1 = 0                                        
                                        if  Ncomp == 1:  
                                                plot(ppmxfd,self.outfdir[:,i1],clr[0], linewidth=2)
                                        else:
                                                for cl in clr[:Ncomp]:
                                                        if self.outfdir[:,i1].max()==0:
                                                                plot(ppmxfd,self.outfdir[:,i1],cl, linewidth=2)
                                                        else:
                                                                plot(ppmxfd,self.outfdir[:,i1]/self.outfdir[:,i1].max(),cl, linewidth=2)                                                
                                                        i1 = i1+1
                                        legend(lst, loc =(1.1,0.5), handlelen = 0.03,prop=font)					
                                        axis('tight')
                                        ylabel(self.tpfdir[0],rotation=0)
                                        xlim((max(ppmxfd),min(ppmxfd)))
			show()

        def PlotFmany(self, event=None):			
                        clr, pvalue, Ncomp, lst, font, lnf, xfd, xf = self.PlotParam()
			# for each row (nucleus)			
			param = 9
			IniSize = gcf().get_size_inches()			
                        for k in range(Ncomp):                                
				figure(num=k+1, figsize = (IniSize[0], IniSize[1]))
                                for n in range(self.ParMx.shape[0]):
                                        if lnf>param: 
                                        	if n>param-1: 
                                                	figure(num=Ncomp+k+1, figsize = (IniSize[0],IniSize[1]))
                                                	pn = str(lnf-param)+str(1)+str(n-param+1)
                                        	else: pn = str(param)+str(1)+str(n+1)
                                	else: pn = str(lnf)+str(1)+str(n+1)
                                        subplot(pn)
                                        subplots_adjust(right=0.7)
                                        if (n):
                                                if logical_and(self.ParMx[n,0]!=0., self.ParMx[n,1]!=0):
                                                        ppmxf = [x*(-self.ParMx[n,0]/(self.outf.shape[0]-1))+self.ParMx[n,1]+self.ParMx[n,0]/2. for x in xf]
                                                else: ppmxf=list(xf)
                                                plot(ppmxf,self.outf[:,n-1,k],clr[k],linewidth=2)
                                                ylabel(self.tpf[n-1],rotation=0)
                                                xlim((max(ppmxf),min(ppmxf)))
                                        else:
                                                if logical_and(self.ParMx[n,0]!=0., self.ParMx[n,1]!=0):
                                                        ppmxfd = [x*(-self.ParMx[n,0]/(self.DirSize-1))+self.ParMx[n,1]+self.ParMx[n,0]/2. for x in xfd]
                                                else: ppmxfd=list(xfd)
                                                plot(ppmxfd,self.outfdir[:,k],clr[k], linewidth=2, label = lst[k])										
                                                legend(loc =(1.1,0.5), handlelen = 0.03,prop=font)						
                                                axis('tight')
                                                ylabel(self.tpfdir[0],rotation=0)
                                                xlim((max(ppmxfd),min(ppmxfd)))                                
                	show()

#===== MAIN Program
# additional module
from IOprodec import *
# standart modules
from pylab import *
from Tkinter import *
from tkFileDialog import *
from xml.dom.minidom import *
from matplotlib.font_manager import fontManager, FontProperties
from numpy import *
import tkMessageBox, glob, os, shutil  

if __name__ == "__main__":        
        root = Tk()        
        app = guiPRODECOMP(root)
        root.withdraw()
        #for Windows
        #if "idlelib" not in sys.modules:                
        root.mainloop()
