"""
======================COPYRIGHT/LICENSE START==========================

guicomp2assign.py: Graphical user interface for SHABBA program

Copyright (C) 2009 Doroteya K. Staykova and Martin Billeter 
(University of Gothenburg, Sweden)

=======================================================================

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
======================COPYRIGHT/LICENSE END============================

for further information, please contact :
- our website (http://www.lundberg.gu.se/nmr/)
- dkstaykova@gmail.com, martin.billeter@chem.gu.se 

=======================================================================

If you are using this software for academic purposes, we suggest
quoting the following reference:

===========================REFERENCE START=============================
Staykova, D. K.; Fredriksson, J.; Bermel, W. and Billeter, M. (2008) 
Assignment of protein NMR spectra based on projections, multi-way decomposition 
and a fast correlation approach. J. Biomol. NMR., 42, 87-97.

===========================REFERENCE END===============================
"""

from pylab import *
from numpy import *
from Tkinter import *
import tkFileDialog, tkMessageBox
import glob, os, time
	
from LoadComp import LoadComp
from readseq import comp2seq
from MatchCaCb import FuncCompar
from ExtractCaCb import FuncCaCb
from MatchHaHb import FuncHaHb
from PeakPickShabba import Peak4Shabba
from FitShape import fitshape

# Path to a file with peak picking parameters
dirin = os.getcwd().split(os.sep+'ProjTools')[0]+os.sep+'ProjTools'+os.sep+'shabba'+os.sep

class comp2bba (object):
        def __init__(self, master):
                self.master = master		
                self.TL = Toplevel(master, pady = 10, padx = 60)
                self.TL.title("SHABBA (SHape Analysis for BackBone Assignments)")

		Button(self.TL,text='Load selected PRODECOMP results', bg = "yellow", \
			command=self.loadPkList).grid(column=0, columnspan=2, sticky='nswe')		
			
		# List indirect dimensions for SHABBA
		frm = Frame(self.TL, relief=RIDGE, bd=2)
		frm.grid(column=0, columnspan=2, sticky='nswe')		
		Ltpf = ['Cabi-1', 'Habi-1', 'Ca', 'Cb', 'Ha', 'Hb']
		Coltpf = ['3','4','5','6','7','8']		
		self.param = [] 
		for i in range(len(Ltpf)):
			f = Frame(frm)
			f.grid(column = i, row=3)
			Label(f,width=8, text=Ltpf[i]).grid()
			e = Entry(f, width=6, bg='white', relief=RIDGE)
                        e.grid()
                        e.insert(END, Coltpf[i])
			e.config(state=DISABLED)
			self.param.append(e)

		self.ButGly = Button(self.TL, text='Check for Gly', bg = "light yellow", disabledforeground = "black", command=self.compGly)
		self.ButGly.grid(row=2, column=0, sticky='nswe')
		self.ButGly.config(state=DISABLED)
		self.ButNs = Button(self.TL, text='Correlate shapes', bg="dark gray", fg = "white", disabledforeground = "white",command=self.compCorr)
		self.ButNs.grid(row=2, column=1, sticky='nswe')
		self.ButNs.config(state=DISABLED)
		
	#==Function for counting number of peaks with intensity > treshold==                
        # An-> column vector, np-> number of points 
	def pkpicker(self, vec, tresh):
		    An = vec.copy().reshape(-1,1)    
                    np = An.shape[0]
                    for i in range(np):
                        if An[i] <= tresh*An.max():
                           An[i] = 0 
		    # Counting peaks above treshold
		    Npks = 0
		    for k in range(np):
			if An[k]:
				if k not in [0,np-1]:
					if all([An[k]>An[k+1],An[k]>An[k-1]]): Npks=Npks+1
				elif k:
					if An[k]>An[k-1]: Npks=Npks+1
				else:
					if An[k]>An[k+1]: Npks=Npks+1		    
		
                    return Npks	

	def compGly(self, event=None):
		#===Take only the shapes for i-residue===		
		ColSh = [int(x.get())-1 for x in self.param[2::]]
		# Ca/Cb and Ha/Hb shapes are examined for Gly component, i.e. if Ca/Ha is in the shape of Cb/Hb		
		# Ratio of shape Ca/Cb and Ha/Hb that is <<1 (RFactor) for Gly or noise in Ca/Ha (many peaks with I>50%Imax)
		#RFactor=0.1
		#RFactor = 0.15
		rC, rH, glyC, glyH, gly = [], [], [], [], []
		try:
			Ncomp = self.Lindshp[0].shape[0]		
                        if all([x!=-1 for x in ColSh[0:2]]):
                                rC = []
                                for x in range(Ncomp):
                                        if logical_and(float(self.Lindshp[ColSh[0]][x,:].max()), float(self.Lindshp[ColSh[1]][x,:].max())):
                                                rC.append(float(self.Lindshp[ColSh[0]][x,:].max())/float(self.Lindshp[ColSh[1]][x,:].max()))
                                        else: rC.append(0)
				
				NpkCA, NpkCB = [], []				

				for x in range(Ncomp):
					if self.Lindshp[ColSh[0]][x,:].any()!=0:
						NpkCA.append(self.pkpicker(self.Lindshp[ColSh[0]][x,:],0.5))
					else: 
						tkMessageBox.showerror(title=None, message='Ca for component '+ str(x)+' has only zeros!')	
				
					if self.Lindshp[ColSh[1]][x,:].any()!=0:
						NpkCB.append(self.pkpicker(self.Lindshp[ColSh[1]][x,:],0.5))						
					else:
						tkMessageBox.showerror(title=None, message='Cb for component ' + str(x)+' has only zeros!')
						
				addCompC = []
				for j,x in enumerate(NpkCA):
					# condition: if many peaks found in Ca shape (likely noise), and few in Cb (unlikely noise)
					if all([x>3, NpkCB[j]<=3]): addCompC.append(j)
					
				# statistics on rC: hist(rC,15)-> shows rC as lognormal distribution
				# Cutoff -> values below 2 standard deviations from the mean of the log(rC)
				LogrC = [log(x) for x in rC if x!=0]
				CutoffC = exp(mean(LogrC)-2*std(LogrC))	
				GcheckC = []				
				for count, x in enumerate(rC):
					if logical_and(x<CutoffC,x!=0): GcheckC.append(count)					
                                glyC = GcheckC + [y for y in addCompC if y not in GcheckC]
                                f = open(self.flog,'a')
                                f.write(time.ctime(time.time())+' Ca/Cb shapes have been examined for Gly component.\n')
                                if len(glyC):
                                        f.write('The following components are found as Gly:\n')
                                        [f.write(str(x)+'\n') for x in glyC]
                                f.close()
                        
                        if all([x!=-1 for x in ColSh[2::]]):
                                rH = []
                                for x in range(Ncomp):
                                        if logical_and(float(self.Lindshp[ColSh[2]][x,:].max()), float(self.Lindshp[ColSh[3]][x,:].max())):
                                                rH.append(float(self.Lindshp[ColSh[2]][x,:].max())/float(self.Lindshp[ColSh[3]][x,:].max()))
                                        else: rH.append(0)

				NpkHA, NpkHB = [], []	
				for x in range(Ncomp):
					if self.Lindshp[ColSh[2]][x,:].any()!=0:
						NpkHA.append(self.pkpicker(self.Lindshp[ColSh[2]][x,:],0.5))
					else: 
						tkMessageBox.showerror(title=None, message='Ha for component '+ str(x)+' has only zeros!')	
					
					if self.Lindshp[ColSh[3]][x,:].any()!=0:
						NpkHB.append(self.pkpicker(self.Lindshp[ColSh[3]][x,:],0.5))
					else:
						tkMessageBox.showerror(title=None, message='Hb for component '+ str(x)+' has only zeros!')
				
				addCompH = []
				for j,x in enumerate(NpkHA):
					# condition: if many peaks found in Ha shape (likely noise), and few in Hb (unlikely noise)
					 if all([x>4, NpkHB[j]<=5]): addCompH.append(j)
				# statistics on rH: Cutoff -> values below 2 standard deviations from the mean of the log(rH)
				LogrH = [log(x) for x in rH if x!=0]
				CutoffH = exp(mean(LogrH)-2*std(LogrH))
				GcheckH = []				
				for count, x in enumerate(rH):
					if logical_and(x<CutoffH,x!=0): GcheckH.append(count)
				glyH = GcheckH + [y for y in addCompH if y not in GcheckH]                                
                                f = open(self.flog,'a')
                                f.write(time.ctime(time.time())+' Ha/Hb shapes have been examined for Gly component.\n')
                                if len(glyH):
                                        f.write('The following components are found as Gly:\n')
                                        [f.write(str(x)+'\n') for x in glyH]
                                f.close()
                        # Collecting the Gly component in one list 'gly' 
                        if len(glyC):
                                gly = list(glyC)
                                [gly.append(x) for x in glyH if len([y for y in glyC if y==x])==0]
                        elif len(glyH):
                                gly = list(glyH)
                        else: tkMessageBox.showinfo(title=None, message='No Gly components found!')
		
		except: 
			tkMessageBox.showerror(title=None, message='An error occurred while reading shapes!')
			self.TL.destroy()
		
		# PLOT of Gly components
		if len(gly):
			#==Function for saving the choice of gly components==
			def ResGly():
				glyshp = [[] for i in range(len(self.entrypar))]
                		for i in range(len(self.entrypar)):			                   
                        		for col in self.entrypar[i]:
                                		glyshp[i].append(int(col.get()),)
				# Components confirmed to be Gly
				glyout = [glyshp[0][x] for x in range(len(glyshp[0])) if glyshp[3][x]==1] 				
				f = open(self.flog,'a')
				f.write(time.ctime(time.time())+' The following components are accepted as Gly and their shapes corrected:\n')
				[f.write(str(x)+'\n') for x in glyout]
				f.close()
				# Correction of Ca, Cb, Ha, Hb shapes of Gly components
				sindx = [int(x.get())-1 for x in self.param[2::]]				
				for comp in glyout:
					if logical_and(sindx[0]!=-1,sindx[1]!=-1):
						self.Lindshp[sindx[0]][comp]=self.Lindshp[sindx[1]][comp].copy()
						self.Lindshp[sindx[1]][comp]=array([0 for x in self.Lindshp[sindx[1]][comp]])
					if logical_and(sindx[2]!=-1,sindx[3]!=-1):						
						self.Lindshp[sindx[2]][comp]=self.Lindshp[sindx[3]][comp].copy()
						self.Lindshp[sindx[3]][comp]=array([0 for x in self.Lindshp[sindx[3]][comp]])
				
				[x.config(state=NORMAL) for x in self.param]			
				self.GlyWin.destroy()
				self.ButGly.config(state=DISABLED)
			
			#==Function for plotting the indirect shapes of a found Gly component==
			def plotshapes():				
				# Readin the acquisition parameters
				pvalue = []
                		for row in self.rows:                        
                        		for col in row:
                                		pvalue.append(col.get(),)
                		k = 0				
                		for i in range(self.ParMx.shape[1]):
                        		for j in range(self.ParMx.shape[0]):
                                		self.ParMx[j,i]=float(pvalue[k])
                                		k = k+1				
				xf = range(len(self.Lindshp[0][int(self.IntNo.get()),:]))
				lnf = len(self.Lindshp)
				IniSize = gcf().get_size_inches()
				figure(num=1, figsize = (IniSize[0],int(IniSize[1]/2)*lnf))
				
				try:
					for n, elem in enumerate([int(x.get())-1 for x in self.param[2::]]):
						pn = str(4)+str(1)+str(n+1)
						subplot(pn)
						if logical_and(self.ParMx[elem,0]!=0., self.ParMx[elem,1]!=0):
                                                	ppmxf = [x*(-self.ParMx[elem,0]/(len(xf)-1))+self.ParMx[elem,1]+self.ParMx[elem,0]/2. for x in xf]
                                        	else:
                                                	ppmxf=list(xf)					
						plot(ppmxf,self.Lindshp[elem][int(self.IntNo.get()),:],linewidth=2)
						ylabel(self.tpf[elem],rotation=0)
						xlim((max(ppmxf),min(ppmxf)))
					show()
					self.PlotTL.destroy()
				except:
					tkMessageBox.showerror(title=None, message='Shapes can not be plot!')
				
			#==Function for opening a window with acquisition parameters==
			def AcqusWin():
                                self.PlotTL = Toplevel(self.GlyWin, pady=10)
                                self.PlotTL.title("Acquisition parameters --> pt-to-ppm") 
                                #yn(ppm)= a*x(pt)+b; a: -SW(ppm)/Np; b: On/BFn + SW(ppm)/2.; OP = On/BFn
                                acquspar = ['Sweep width, ppm\n ','Transmitter frequency offset /\nMiddle of the range, ppm']
                                F0=Frame(self.PlotTL)
                                F0.pack(side=LEFT)
                                LabelDim = self.tpf                                       
                                Label(F0).pack(side=TOP)
                                Label(F0,text=' ').pack(side=TOP)
                                #matrix with acqus parameters
                                self.ParMx = zeros((len(LabelDim),len(acquspar)))                                
				self.ParMx[:,0], self.ParMx[:,1] = [float(x) for x in self.SWppm[1::]], [float(x) for x in self.O1ppm[1::]] 

                                for j in range(len(LabelDim)): Label(F0,text=LabelDim[j]).pack(side=TOP)                        
                                                            
                                                             
                                F1 = Frame(self.PlotTL)
                                F1.pack(side=LEFT)
                                self.rows = []
                                for i in range(self.ParMx.shape[1]):                    
                                    f = Frame(F1)
                                    f.pack(side=LEFT)
                                    Label(f,text=acquspar[i]).pack(side=TOP)  
                                    cols = []                    
                                    for j in range(self.ParMx.shape[0]):                          
                                        e = Entry(f,relief=RIDGE)
                                        e.pack(side=TOP, padx=5)
                                        e.insert(END, self.ParMx[j, i])
                                        cols.append(e)
                                    self.rows.append(cols)
                               
                                Button(self.PlotTL,text='Plot', command=plotshapes).pack(side=BOTTOM)
			
                        self.GlyWin = Toplevel(self.TL, pady = 20)
                        self.GlyWin.title("Gly components")
                        NameCol = ['Comp No\n', 'C-criteria\n ', 'H-criteria\n ','Correct compNo\n1(yes) / 0(no)']
			Fcol = Frame(self.GlyWin, bd = 2, relief=RIDGE)
			Fcol.grid(columnspan=4)			
                        for j in range(len(NameCol)): Label(Fcol,text=NameCol[j], width=15).grid(column=j, row=0)                        
                                             
                        InPar = zeros((len(gly),4))
			InPar[:,0], InPar[:,3] = sorted(gly), len(gly)*[1]
			if (len(glyC)):	
				for k in range(InPar[:,1].shape[0]):
					 if len([x for x in glyC if x==InPar[k,0]])!=0: InPar[k,1]=1
			if (len(glyH)):	
				for k in range(InPar[:,2].shape[0]):
					 if len([x for x in glyH if x==InPar[k,0]])!=0: InPar[k,2]=1
			self.entrypar = [[] for x in range (InPar.shape[1])]
			for i in range(InPar.shape[1]):                    
                    		f = Frame(Fcol)
                    		f.grid(row=1,column=i)
                    		for j in range(InPar.shape[0]):
					if i==InPar.shape[1]-1:                          
                        			e = Entry(f, width = 6,relief=RIDGE)
                        			e.grid()
                        			e.insert(END, int(InPar[j, i]))
                        	
					else:	
						e = Entry(f, width = 6, relief=FLAT, disabledforeground="black")
						e.grid()
						e.insert(END, int(InPar[j, i]))				
						e.config(state=DISABLED)
					self.entrypar[i].append(e)
			
			self.IntNo = StringVar()			
			EntSel = Entry(Fcol, width = 6,relief=RIDGE, textvariable = self.IntNo)
                        EntSel.grid(row=2, column=2, columnspan=2, sticky='nswe')
			Button(Fcol,text='Plot Comp No', bg = "yellow", \
				command = AcqusWin).grid(row=2, columnspan=2, sticky='nswe')
			Button(Fcol,text='Confirm Gly components', bg = "light yellow", \
				command = ResGly).grid(row=3, columnspan=4, sticky='nswe')
	def compCorr(self, event=None):
		#==Function for saving correlation results as an excel file
		# the input is python array 'a' and a file name
		def outascii(a,filename):
                    a = asmatrix(a)
                    [r,c] = a.shape
                    lst = []
                    for i in range(r):
                        l1 = []
                        for j in range(c):
                            if l1 == []: l1 = str(a[i,j])+'\t'
                            else: l1 = l1 + str(a[i,j])+'\t'
                        l1 = l1 + '\n'
                        if lst == []: lst = l1
                        else: lst = append(lst,l1)    
                    f = open(filename, 'w')
                    f.writelines(lst)
                    f.close()
                    return

		#==Function for processing the correlation table
                def processCorTab():		
			if len(self.EntF.get()):				
				MatchBut.config(state=NORMAL)
				self.Tbox.config(state=NORMAL)
				if len(self.Tbox.get(0.0,END).split()): self.Tbox.delete(0.0,END)				
                                Factor = float(self.EntF.get())
                                poutk = self.outk.copy()
								
                                # rule: set diagonals to zero
                                c1 = 0
                                for i in range(poutk.shape[0]):
                                        if poutk[i,i]!=0: 
                                                poutk[i,i] = 0
                                                # count the changes
                                                c1 = c1+1
                                f = open(self.flog,'a')
                                f.write('\n'+time.ctime(time.time())+' Analysis of the correlation results using factor '+self.EntF.get()+' gives:\n')
                                f.write(str(c1)+' changes after setting the diagonal to zero.\n')
                                f.close()
				cSum = c1
				while cSum!=0:
                                        # rule: set element to zero if it is smaller then factor*mirror element with respect to diagonal    
                                        c2 =0
                                        for i in range(poutk.shape[0]):
                                                for j in range(poutk.shape[0]):
                                                        if i!=j:
                                                                if all([poutk[i,j] < Factor*poutk[j,i],poutk[i,j]!=0]):
                                                                        poutk[i,j] = 0
                                                                        # count the changes
                                                                        c2 = c2+1
                                        f = open(self.flog,'a')
                                        f.write(str(c2)+' changes after setting element to zero if it is smaller then '+str(Factor)+'* mirror element.\n')
                                        f.close()
                                        # rule: set element(row, col) to zero if it is smaller then factor*max(row) or factor*max(col)
                                        c3 = 0
                                        for i in range(poutk.shape[0]):
						for k in range(poutk.shape[1]):							
							if poutk[i,k]!=0: 
								if poutk[i,k] < Factor*poutk[i,:].max():
									poutk[i,k]=0
                                                			c3 = c3+1
                                                
                                        for j in range(poutk.shape[0]):
    						for k in range(poutk.shape[1]):							 
							if poutk[k,j]!=0:
								if poutk[k,j] < Factor*poutk[:,j].max():
									poutk[k,j]=0
                                                			c3 = c3+1
                                                	
                                        f = open(self.flog,'a')
                                        f.write(str(c3)+' changes after setting element(row, col) to zero if it is smaller then '\
                                                +str(Factor)+'* max(row) or '+str(Factor)+'* max(col).\n')
                                        f.close()

					# rule: if a row(column) has a single remaining entry (unique entry), then all other entries on the same column(row) as this unique entry are removed
					# provided they are not themselves already unique: a unique entry on a row should also be unique on its column
					c4 = 0
					for i in range(poutk.shape[0]):
						for j in range(poutk.shape[1]):							
							if poutk[i,j]!=0:
								# element (i,j) is unique on its row
								if len([x for x in poutk[i,:] if x!=0])==1:
									for k in [y for y in range(poutk.shape[0]) if y!=i]:
										# check if element (k!=i,j) is not unique
										if poutk[k,j]!=0:
											if len([x for x in poutk[k,:] if x!=0])>1:
												poutk[k,j]=0
												c4 = c4+1
					f = open(self.flog,'a')
                                        f.write(str(c4)+' changes after setting element(row, col) to zero if it is not unique on its row (column).\n')
                                        f.close()

					# number of changes after all rules are applied
					cSum = c1+c2+c3+c4
					c1 = 0
				
                                rpoutk = (poutk*100).round(decimals=1)
                                Find = str(Factor).find('.')			
                                outascii(rpoutk,self.dirout+str(Factor)[:Find]+str(Factor)[Find+1:]+'_corr.xls')
                                # Displaying the resulted chains
                                res_i, res_i_1=[],[]                        
                                for i in range(poutk.shape[0]):
                                        for j in range(poutk.shape[0]):
                                                if poutk[i,j]!=0:            
                                                        #check the row and column
                                                        v1 = len(nonzero(poutk[i,:])[0])
                                                        v2 = len(nonzero(poutk[:,j])[0])            
                                                        if logical_or(v1>1, v2>1): 
								f = open(self.flog,'a')
								f.write('connection '+str(j)+'-'+str(i)+' is ambigous!\n')
								f.close()
                                                        else:
                                                                res_i=append(res_i,i)
                                                                res_i_1=append(res_i_1,j)
                                # The length of the res_i and res_i_1 is the same, Nf
                                # matrix of connected residues, each row: i-1 to i
                                res_mx = concatenate((asmatrix(res_i_1),asmatrix(res_i))).T
                                var_mx = res_mx.copy()
                                fragm = [str(int(var_mx[0,0])),str(int(var_mx[0,1]))]
                                var_mx = delete(var_mx, s_[0:1], axis=0)
                                nfrag = 0
				#== Initial set-up of the component chains
				self.ListChain=[]					
                                while var_mx.shape[0]!=0:					
                                        N = var_mx.shape[0]					
                                        for i in range (N):
                                                if int(fragm[-1])==var_mx[i,0]:
                                                        fragm.append(str(int(var_mx[i,1])))
                                                        var_mx = delete(var_mx, s_[i:i+1], axis=0)
                                                        break
                                                elif int(fragm[0])==var_mx[i,1]:
                                                        fragm.insert(0,str(int(var_mx[i,0])))                                		
                                                        var_mx = delete(var_mx, s_[i:i+1], axis=0)
                                                        break
                                                elif i== N-1:
                                                        nfrag = nfrag+1
							result = 'Chain No'+str(nfrag)+', length='+str(len(fragm))+': '+fragm[0]
							self.ListChain.append(fragm)
							for x in fragm[1::]: result = result+'-'+x						
							self.Tbox.insert(END, result+'\n')
							self.Tbox.insert(END, '\n')
           						f = open(self.flog,'a')
							f.write ('Chain No'+str(nfrag)+', length='+str(len(fragm))+'->')
                                                        [f.write(' '+ x) for x in fragm]
							f.write('\n')
							f.close() 
                                                        fragm = [str(int(var_mx[0,0])),str(int(var_mx[0,1]))]                          		
                                                        var_mx = delete(var_mx, s_[0:1], axis=0)  
				
                                if nfrag != 0:
					result = 'Chain No'+str(nfrag+1)+', length='+str(len(fragm))+': '+fragm[0]
					for x in fragm[1::]: result = result+'-'+x
					self.ListChain.append(fragm)					
					self.Tbox.insert(END, result+'\n')
					self.Tbox.insert(END, '\n')
					
 					f = open(self.flog,'a')
					f.write ('Chain No'+str(nfrag+1)+', length='+str(len(fragm))+'->')
					[f.write(' '+ x) for x in fragm]
					f.write('\n')
					f.close()
				# Check if every component is included in a chain: if not, lists in a separate chain
				AddFragm = []
				for i in range(poutk.shape[0]):
					if len([str(i) for elem in self.ListChain if str(i) not in elem])==len(self.ListChain):
						AddFragm.append([str(i)]) 
				
				if len(AddFragm):
					for fr in AddFragm: 
						self.ListChain.append(fr)
						result = 'Chain No'+str(len(self.ListChain))+', length=1: '+fr[0]
						self.Tbox.insert(END, result+'\n')
						self.Tbox.insert(END, '\n')
						f = open(self.flog,'a')
						f.write ('Chain No'+str(len(self.ListChain))+', length=1->')
                                		[f.write(' '+ x) for x in fr]
						f.write('\n')
						f.close() 
				self.Tbox.config(state=DISABLED)
                        else: tkMessageBox.showwarning(title=None, message='Please enter Factor value!')
      		
		#==Function for positioning chains in a protein sequence
                def sh2seq():			
			#==takes shapes that correspond to Cab(i-1), Ca, Cb
			#!Assumes that the shapes are available and are not zeros
			ColInd = [int(self.param[x].get())-1 for x in [0,2,3]]
			LCshp = [self.Lindshp[x] for x in ColInd]			
			#==chemical shifts difference (self.Cdelta) between a peak in 'i' and 'i-1' shape: missmatch(pt)*[SW/Npts]+[SW/Npts]/2.
			SW = float(self.SWppm[int(self.param[0].get())])
			Cppm_pt =SW/self.Lindshp[0].shape[1]			
			self.Cdelta = float(self.PPar[0].get())*Cppm_pt+Cppm_pt/2.
			tresh = float(self.PPar[1].get())			
			MAX = float(self.O1ppm[int(self.param[0].get())])+SW/2.
								
			# Generate lists of peaks & intensities <-> lists of shapes
			PeakShape = [[] for i in range(len(LCshp))]	# peaks in ppm
			IntShape = [[] for i in range(len(LCshp))]	# intensities of peaks
			for i, TypeShape in enumerate(LCshp):
				for shp in TypeShape:					
					result = Peak4Shabba(shp,SW, MAX)
					PeakShape[i].append(result[1])
					IntShape[i].append(result[2])
			# Extracting (Cai-1,Cbi-1,Cai,Cbi) for each component 
			MatchCab = []
			for i in range(len(PeakShape[0])):
				InPk, InInt = [PeakShape[x][i] for x in range(3)],[IntShape[y][i] for y in range(3)]  
				MatchCab.append(FuncCompar(InPk, InInt, self.Cdelta, tresh))
			
			# Comparing (Cai-1,Cbi-1,Cai,Cbi) of neighbouring components
			compCa = [[] for i in range(len(self.ListChain))] 
			compCb = [[] for i in range(len(self.ListChain))]
			for k, ch in enumerate(self.ListChain):
				if len(ch)==1: # chain of 1 component
					for i in [0,2]:
						if MatchCab[int(ch[0])][i]!=[]: compCa[k].append(MatchCab[int(ch[0])][i])
						else: compCa[k].append(0)
					for j in [1,3]:
						if MatchCab[int(ch[0])][j]!=[]: compCb[k].append(MatchCab[int(ch[0])][j])
						else: compCb[k].append(0)
					
				else:
					for i, comp in enumerate(ch):
						if not i: # first component in a chain
							if MatchCab[int(comp)][0]!=[]: compCa[k].append(MatchCab[int(comp)][0])
							else: compCa[k].append(0)
							if MatchCab[int(comp)][1]!=[]: compCb[k].append(MatchCab[int(comp)][1])
							else: compCb[k].append(0)
							result = FuncCaCb(MatchCab[int(comp)], MatchCab[int(ch[i+1])], self.Cdelta)
							compCa[k].append(result[0])
							compCb[k].append(result[1])
						
						elif i!=len(ch)-1: 	  # within the chain (compare with the next component)
							result = FuncCaCb(MatchCab[int(comp)], MatchCab[int(ch[i+1])], self.Cdelta)
							compCa[k].append(result[0])
							compCb[k].append(result[1])
						else: 	# last component
							if MatchCab[int(comp)][2]!=[]: compCa[k].append(MatchCab[int(comp)][2])
							else: compCa[k].append(0)
							if MatchCab[int(comp)][3]!=[]: compCb[k].append(MatchCab[int(comp)][3])
							else: compCb[k].append(0)
							
			# compCb, compCa, ListComp-> list of chains(lists)
			self.assignComp, self.assignCaCb, self.NPhalpha, self.NPhbeta, self.seq = comp2seq(20,self.ListChain, compCb, compCa, self.dirout)
			# Calculate statistics on rmsd of the assigned residues (find outliers/wrong assignments)
			RMSD = [float(x[-1]) for x in self.assignComp]
			RmsdTresh = mean(RMSD)+2*std(RMSD)
			Indic = []
			for i in range(len(RMSD)):			
				if any([RMSD[i]>=RmsdTresh, RMSD[i-1]>=RmsdTresh]): Indic.append('questionable')
				else: Indic.append('--')
			self.warning = [[] for x in range(len(self.seq))]
			for i, elem in enumerate(self.seq):
				val = [y for y in range(len(self.assignComp)) if self.assignComp[y][1]==str(i+1)+elem]
				if len(val):
					self.warning[i]=Indic[val[0]]				
			#==Output the assignement
			f = open(self.flog,'a')
			f.write('\n'+time.ctime(time.time())+' The obtained assignment is:\n')
			for res in self.assignComp:	
				[f.write(str(k)+' ') for k in res]
				f.write('\n') 
			f.close()
			SaveRes.config(state=NORMAL)			 
			return			

		def SaveBBA():
			MatchBut.config(state=DISABLED)	
			# Preparing the ND peak list+assignment: compNo, ResNo, ChemShifts: HN, N, CO (own), Ca, Cb, Ha, Hb
			# !Only unique answers are assigned
			#==function for peak picking shapes that are not Ca/Cb
                        def OutputNDpks(LIndShp):
				Loffset = [int(x) for x in self.offset]
                                # fill in the peak values for all shapes -> ND peak list:[for all nuclei [for all components [for all peaks]]]
                                NDPeakList = [[[] for y in range(len(self.Ldshp))] for x in range(len(self.Nuc))] # peak positions
                                NDInt = [[[] for y in range(len(self.Ldshp))] for x in range(len(self.Nuc))] # peak intensity		
                                params = [[] for y in range(len(self.Nuc))] # peak picking parameters
				
				# Opening a parameter file with the rest of peak picking parameters		
                                nuc, Npk = [], []
                                try:					
                                        f = open(dirin+'ShabbaPar.txt','r')
                                        buf = f.readlines()
                                        f.close()
                                        for i in range(1, len(buf)):
                                                if len(buf[i].split()):
                                                        nuc.append(buf[i].split()[0])                                                        
                                                        Npk.append(buf[i].split()[1]) # it can be '>2'->string type
                                        # Select entries dependig on the present nuclei types
                                        SelNpk = []
                                        for k in range(len(self.Nuc)):
                                                match = [x for x in nuc if x==self.Nuc[k]]
                                                if (len(match)==1):			
                                                        imatch = nuc.index(match[0])
                                                        SelNpk.append(Npk[imatch])
                                                        
                                                else: tkMessageBox.showwarning(title=None, message='Nuclei '+match[0]+' is repeated in ShabbaPar.txt!')
                                        
                                        for m in range(len(self.Nuc)):
                                                params[m]=[self.SW[m], self.MAX[m], SelNpk[m]]
                                        
                                        for i, elem in enumerate(self.Nuc):                                        
                                                SW, MAX = self.SW[i], self.MAX[i]                                                
                                                Npks = params[i][2] # number of peaks to be taken starting from the strongest one                                        
                                                
                                                if not i: ShpMx = self.Ldshp
                                                else: ShpMx = LIndShp[i-1].tolist()
                                                for j, Scomp in enumerate(ShpMx):
                                                        if self.Nuc[i]!='HN': 	# Indirect shapes: can have more then 1 peak
                                                                PksOut = Peak4Shabba(Scomp,SW, MAX)					
                                                                if all([ord(Npks[0])>= 48, ord(Npks[0]) <= 57]):
                                                                                SelectInt = sorted(PksOut[2], reverse=True)[0:int(Npks)] # Selecting the Npks number of strongest peaks 
                                                                                SelIndx = [x for x in range(len(PksOut[2])) if PksOut[2][x] in SelectInt] 
                                                                                NDPeakList[i][j]=[PksOut[1][k] for k in SelIndx]
                                                                                NDInt[i][j]=[PksOut[2][k] for k in SelIndx]
                                                                else: 
                                                                        NDPeakList[i][j]=PksOut[1] # If number of expected peaks is not fixed, take them all
                                                                        NDInt[i][j]=PksOut[2]
                                                        else: 
                                                                NDPeakList[i][j] = fitshape(Scomp, int(self.Size[0]), Loffset[j], 'G', SW, MAX)[1]	# Direct shape: only one peak is expected
                                                                NDInt[i][j] = [100.0]
                                        return	NDPeakList, NDInt
                                                        
				except:					
					tkMessageBox.showerror(title=None, message='Error reading parameters from \'ShabbaPar.txt\'!')
					return None				
                                


			ColIndND= [x for x in range(len(self.tpf)) if self.tpf[x] not in ['Cabi-1','Ca', 'Cb']]
			LCshp = [self.Lindshp[x] for x in ColIndND]
			self.Nuc = [self.NucType[0]]+[self.tpf[x] for x in ColIndND]
			self.SW = [float(self.SWppm[0])]+[float(self.SWppm[1::][x]) for x in ColIndND]
			self.O1 = [float(self.O1ppm[0])]+[float(self.O1ppm[1::][x]) for x in ColIndND]
			self.MAX = [self.O1[x]+self.SW[x]/2. for x in range(len(self.O1))]
			
			# List of shifts for each component, order is according the order of components
			if OutputNDpks(LCshp)!= None: NDPos, NDIntens = OutputNDpks(LCshp)
		
			self.ChemShift = ['HN','N','C','CA', 'CB', 'HA1','HA2','HB2', 'HB3']			
			self.CSList = zeros((len(self.seq), len(self.ChemShift)))
			#==Filling the Ca & Cb shifts
			for i in range(len(self.seq)):
				loc = [x for x in range(len(self.assignCaCb[0])) if str(i+1)+self.seq[i]==self.assignCaCb[0][x]]
				for j, elem in enumerate(self.ChemShift):				
					if len(loc): # each residue appears only once (see assignCaCb in readseq.py)
						if all([elem=='CA',self.assignCaCb[1][loc[0]]]): self.CSList[i,j] = self.assignCaCb[1][loc[0]]							
						elif all([elem=='CB',self.assignCaCb[2][loc[0]]]): self.CSList[i,j] = self.assignCaCb[2][loc[0]]
						else: self.CSList[i,j]=999.000
					else: self.CSList[i,j]=999.000
			#==Filling HN, N and CO shifts:!Important: self.ChemShift and self.tpf should have the same order of nuclei
			SelNuc = [x for x in range(len(self.Nuc)) if self.Nuc[x] not in ['Habi-1', 'Ha', 'Hb']]
			for ind in SelNuc:
				for j, cs in enumerate(NDPos[ind]):
					if all([len(cs)==1,cs[0]]): # Takes only unambigous shifts
						ln = [x[1] for x in self.assignComp if 'comp'+str(j)==x[0]][0]
						loc = [x for x in range(len(self.seq)) if str(x+1)+self.seq[x]==ln][0]
						if self.Nuc[ind].find('C')==-1: self.CSList[loc, ind] = cs[0]
						else: self.CSList[loc-1, ind] = cs[0] # CO shift of one component is assigned to i-1 residue

			#==Extracting (Hai-1,Hbi-1,Hai,Hbi) for each component using peak lists and components' assignment			
			LeftNuc = [x for x in range(len(self.Nuc)) if self.Nuc[x] in ['Habi-1', 'Ha', 'Hb']]
			H_NDPos = [NDPos[x] for x in LeftNuc]
			H_NDInt = [NDIntens[x] for x in LeftNuc]

			#==chemical shifts difference between a peak in 'i' and 'i-1' shape: missmatch(pt)*[SW/Npts]+[SW/Npts]/2.
			Hppm_pt = self.SW[LeftNuc[0]]/LCshp[0].shape[1]
			self.CSdiff= float(self.delataH.get())*Hppm_pt+Hppm_pt/2.			
			  
			self.MatchHab, self.IntHab = [], []
			self.IntStat = []			
			for i in range(len(H_NDPos[0])): 
					Peaks = [H_NDPos[x][i] for x in range(len(LeftNuc))]
					Intens = [H_NDInt[x][i] for x in range(len(LeftNuc))]
					# 'i' is component number
					Icomp = [self.assignComp[x][1] for x in range(len(self.assignComp)) if self.assignComp[x][0]=='comp'+str(i)][0]
					npHa = [self.NPhalpha[x] for x in range(len(self.NPhalpha)) if str(x+1)+self.seq[x]==Icomp][0]
					npHb = [self.NPhbeta[x] for x in range(len(self.NPhbeta)) if str(x+1)+self.seq[x]==Icomp][0]
					I_1comp = [str(x)+self.seq[x-1] for x in range(len(self.seq)) if str(x+1)+self.seq[x]==Icomp][0]					
					npHai_1 = [self.NPhalpha[x] for x in range(len(self.NPhalpha)) if str(x+1)+self.seq[x]==I_1comp][0]
					npHbi_1 = [self.NPhbeta[x] for x in range(len(self.NPhbeta)) if str(x+1)+self.seq[x]==I_1comp][0]
					Npeaks = [npHai_1, npHbi_1, npHa, npHb]
					self.MatchHab.append(FuncHaHb(Peaks, Intens, Npeaks, self.CSdiff)[0])
					self.IntHab.append(FuncHaHb(Peaks, Intens, Npeaks, self.CSdiff)[1])
					[[self.IntStat.append(y) for y in x] for x in FuncHaHb(Peaks, Intens, Npeaks, self.CSdiff)[1] if x!=[]]
			
			# Looking at the statistics of intensity of Ha/Hb-peaks of i&i-1 residues to determine a treshold			
			self.StatWin = Toplevel(self.master, pady = 10, padx = 60)
			self.StatWin.title("Statisitcs of Ha/Hb-intensities")
			Button(self.StatWin, text = 'Histogram plot', bg='light yellow', command = PlotStat).grid(row=1, column=0, columnspan=2, sticky ='we')	

			LCount = hist(self.IntStat,bins=range(0,100,10))[0].tolist()
			LValue = range(0,100,10) # left edges of the bins
			# Treshold is defined as the center of the bin that is next to the one where LCount is min, 
			# i.e. left_edge(10)+len_of_bin(10)+len_bin/2.
			self.ITresh = [LValue[x] for x in range(len(LCount)) if LCount[x]==min(LCount)][0]+10+10/2.
			
			self.IntTresh = StringVar()
			self.tresh_ent = Entry(self.StatWin, relief=RIDGE, width=8, bg='white', textvariable = self.IntTresh)
			self.tresh_ent.grid(row=2, column=0)
			self.tresh_ent.insert(END,str(self.ITresh))			
					
			self.TakeBut = Button(self.StatWin, text = 'Take treshold', bg='yellow', command = WriteBBA)
			self.TakeBut.grid(row=2, column=1, sticky ='we')
			self.TakeBut.config(state=DISABLED)
		
		# Plotting statistics: histogram of 10 bins
		def PlotStat():				
			self.TakeBut.config(state=NORMAL)
			figure(num=1)
			hist(self.IntStat,10)
			title('Intensity treshold='+str(self.ITresh)+'%')			
			figure(1).show()			

		def WriteBBA():			
			self.StatWin.destroy()			
			#==Filling Ha, Hb shifts: assuming the order is 'HA1','HA2','HB2', 'HB3'
			HInd = [x for x in range(len(self.ChemShift)) if self.ChemShift[x] in ['HA1','HA2','HB2', 'HB3']]
			N=len(self.seq)
			for i in range(N):
				compL, compR = [],[]
				varL = [self.assignComp[x][0] for x in range(len(self.assignComp)) if self.assignComp[x][1]==str(i+1)+self.seq[i]]
				if len(varL): compL = varL[0]
				
				if not i==N-1: 					
					varR = [self.assignComp[x][0] for x in range(len(self.assignComp)) if self.assignComp[x][1]==str(i+2)+self.seq[i+1]]
					if len(varR): compR = varR[0]
					if all([compL!=[],compR!=[]]): 
						NcompL, NcompR = int(compL.split('comp')[1]), int(compR.split('comp')[1])
						#==HA1, HA2
						if self.NPhalpha[i]==1:
							if all([self.MatchHab[NcompL][2]!=[],self.MatchHab[NcompR][0]!=[]]): 
								if all([abs(self.MatchHab[NcompL][2][0]-self.MatchHab[NcompR][0][0])<self.CSdiff,self.IntHab[NcompL][2][0]>self.IntTresh]): self.CSList[i,HInd[0]]= self.MatchHab[NcompL][2][0] # HA1 takes the own shift
								elif self.IntHab[NcompR][0][0]>self.IntTresh: self.CSList[i,HInd[0]]= self.MatchHab[NcompR][0][0] # HA1 takes i-1 shift as more reliable (own shape can have missing or foreign peaks)
							elif self.MatchHab[NcompL][2]!=[]:
								if self.IntHab[NcompL][2][0]>self.IntTresh: self.CSList[i,HInd[0]]= self.MatchHab[NcompL][2][0]
								
							elif self.MatchHab[NcompR][0]!=[]:
								if self.IntHab[NcompR][0][0]>self.IntTresh: self.CSList[i,HInd[0]]= self.MatchHab[NcompR][0][0]
						else: 
							for j in range(self.NPhalpha[i]):
								if self.MatchHab[NcompR][0]!=[]:
									try:
										if self.IntHab[NcompR][0][j]>self.IntTresh: self.CSList[i,HInd[j]]=self.MatchHab[NcompR][0][j] # i-1 for Gly more reliable
									except:
										pass
								elif self.MatchHab[NcompL][2]!=[]:
									try:
										if self.IntHab[NcompL][2][j]>self.IntTresh: self.CSList[i,HInd[j]]=self.MatchHab[NcompL][2][j] # in case i-1 shifts are missing
									except:
										pass
						#==HB2, HB3
						if self.NPhbeta[i]: 
							for j in range(self.NPhbeta[i]):
								if self.MatchHab[NcompR][1]!=[]:
									try: 
										if self.IntHab[NcompR][1][j]>self.IntTresh: self.CSList[i,HInd[2+j]]=self.MatchHab[NcompR][1][j] # HBi-1 more reliable
									except:
										pass
								elif self.MatchHab[NcompL][3]!=[]:
									try: 
										if self.IntHab[NcompL][3][j]>self.IntTresh: self.CSList[i,HInd[2+j]]=self.MatchHab[NcompL][3][j] # in case i-1 shifts are missing
									except:
										pass
									

					elif compR!=[]:
						Ncomp = int(compR.split('comp')[1])
						if self.NPhalpha[i]==1:
							if all([len(self.MatchHab[Ncomp][0])==1, self.MatchHab[Ncomp][0]!=[]]): 
								if self.IntHab[Ncomp][0][0]>self.IntTresh: self.CSList[i,HInd[0]]= self.MatchHab[Ncomp][0][0] # HA1 unambigous
							for j in range(self.NPhbeta[i]):
								try: 
									if self.IntHab[Ncomp][1][j]>self.IntTresh: self.CSList[i,HInd[j+2]]=self.MatchHab[Ncomp][1][j]
								except:
									pass
						elif self.NPhalpha[i]==2: # Gly as i-1
							if len(self.MatchHab[Ncomp][0])==2: 
								if self.IntHab[Ncomp][0][0]>self.IntTresh: self.CSList[i,HInd[0]]= self.MatchHab[Ncomp][0][0] # HA1 
								if self.IntHab[Ncomp][0][1]>self.IntTresh: self.CSList[i,HInd[1]]= self.MatchHab[Ncomp][0][1] # HA2 
					elif compL!=[]: 
						Ncomp = int(compL.split('comp')[1])
						for j in range(self.NPhalpha[i]):
							try:
								if self.IntHab[Ncomp][2][j]>self.IntTresh: self.CSList[i,HInd[j]]= self.MatchHab[Ncomp][2][j]
							except:
								pass
						for j in range(self.NPhbeta[i]):
							try:
								if self.IntHab[Ncomp][3][j]>self.IntTresh: self.CSList[i,HInd[2+j]]= self.MatchHab[Ncomp][3][j]
							except:
								pass
				else:# last residue
					if compL!=[]:
						Ncomp = int(compL.split('comp')[1])
						for j in range(self.NPhalpha[i]):
							try:
								if self.IntHab[Ncomp][2][j]>self.IntTresh: self.CSList[i,HInd[j]]= self.MatchHab[Ncomp][2][j]
							except:
								pass
						for j in range(self.NPhbeta[i]):
							try:
								if self.IntHab[Ncomp][3][j]>self.IntTresh: self.CSList[i,HInd[2+j]]= self.MatchHab[Ncomp][3][j]
							except:
								pass

						
								
			#==Output ND list
			f = open(self.dirout+'BBAssignment.txt','w')
			hdr = 'Residue\t'
			for x in self.ChemShift: hdr = hdr+x+'\t'
			hdr = hdr+'Warning'+'\n'
			
			data = str(1)+self.seq[0]+'\t'+str(round(self.CSList[0,0],3))+'\t'
			for k in self.CSList[0,1::]: data = data+str(round(k,3))+'\t'

			if len(self.warning[0]): data = data+ self.warning[0]+'\n'
			else: data = data+'\n'

			for i in range(1, self.CSList.shape[0]):
				data = data+str(i+1)+self.seq[i]+'\t'
				for j in range(self.CSList.shape[1]):
					data = data+str(round(self.CSList[i,j],3))+'\t'
				if len(self.warning[i]): data = data+self.warning[i]+'\n'
				else: data = data+'\n'
						
			f.write(hdr+data)			
			f.close()
			tkMessageBox.showinfo(title=None, message='The assignment is stored in '+os.sep+self.dirout+'BBAssignment.txt and the program exits.')	
			self.TL.destroy()					

		#==start of compCorrfloat(EntF.get())
		self.ButNs.config(state=DISABLED)
		MxCab, MxHab, kfCab, kfHab, outkCab, outkHab, outp = [],[],[],[],[], [], []
		if all([self.param[0].get()!='-1', self.param[2].get()!='-1', self.param[3].get()!='-1']): 
			# correlating Ca Cb shapes of i and i-1 residue
			MxCab = concatenate((self.Lindshp[int(self.param[0].get())-1],\
				self.Lindshp[int(self.param[2].get())-1]+self.Lindshp[int(self.param[3].get())-1]))
			kfCab = corrcoef(MxCab)
			# matrix with correlations is a squire
			outkCab = kfCab[:kfCab.shape[1]/2,kfCab.shape[1]/2:kfCab.shape[1]]			
		if all([self.param[1].get()!='-1', self.param[4].get()!='-1', self.param[5].get()!='-1']): 
			# correlating Ha Hb shapes of i and i-1 residue
			MxHab = concatenate((self.Lindshp[int(self.param[1].get())-1],\
				self.Lindshp[int(self.param[4].get())-1]+self.Lindshp[int(self.param[5].get())-1]))
			kfHab = corrcoef(MxHab)
			outkHab = kfHab[:kfHab.shape[1]/2,kfHab.shape[1]/2:kfHab.shape[1]]
		if all([len(outkCab),len(outkHab)]):
			self.outk = (outkCab + outkHab)/2.
			outp = (100*self.outk).round(decimals=1)
		elif len(outkCab): 
			self.outk = outkCab
			outp = 100*self.outk.round(decimals=1)
		elif len(outkHab):
			self.outk = outkHab
			outp = 100*self.outk.round(decimals=1)
		else: 
			self.outk = []
			tkMessageBox.showinfo(title=None, message='No correlation is calculated!')
		# Saving the raw data in an output file
		outascii(outp, self.dirout+'rawcorr.xls')
                f = open(self.flog,'a')
                f.write(time.ctime(time.time())+' Raw correlation results are saved in '+self.dirout+'raw_corr.xls\n')
                f.close()
		if len(outp):
                        # Processing the correlation table
			Label(self.TL, text = '\n').grid()
			WinFragm = Frame(self.TL)
			WinFragm.grid(column=0, columnspan=2)
                        FrmFac = Frame(WinFragm)
			FrmFac.grid(column=0, columnspan=3, sticky='nswe')
			Label(FrmFac, text='Factor').grid(column=1, sticky='nswe')
			self.EntF = StringVar()			
			entfactor = Entry(FrmFac,relief=RIDGE, width = 10, bg='white', textvariable = self.EntF)
                        entfactor.grid(row=0, column=2, sticky='nswe')
			Button(FrmFac, text = 'Process', bg='light yellow', command=processCorTab).\
				grid(row=0, column=3, sticky='nswe')
			sbar = Scrollbar(WinFragm, orient=VERTICAL)			
                	self.Tbox = Text(WinFragm, width = 50,bg = 'white', yscrollcommand=sbar.set)			
                	sbar.config(command=self.Tbox.yview)			
                	sbar.grid(column=3, row=1, sticky='nswe')			
                	self.Tbox.grid(column=0, columnspan=3, row=1, sticky='nswe')
			self.Tbox.__setitem__('height', int(float(self.Tbox.cget('height'))/2))
			FPPpar = Frame(WinFragm, relief=RIDGE, bd=2)
			FPPpar.grid(row=3, column=0, columnspan=2)
			
			# Initial values for C-shapes peak picking: mismatch of C-peaks in (i-1) and (i) shapes; intensity treshold
			IniVal = [1, 5.0]
			self.PPar = []
			for i, elem in enumerate(['C-shift(pt)','Tresh(%)']): 				 
				f = Frame(FPPpar)
				f.grid(column = i, row=3)
				Label(f,width=10, text=elem).grid()
				e = Entry(f, width=10, bg='white', relief=RIDGE)
                        	e.grid()
                        	e.insert(END, IniVal[i])
				self.PPar.append(e)
			
			tkMessageBox.showinfo(title=None, message='Please check peak picking parameters (bottom)!')

			FFinal = Frame(WinFragm)
			FFinal.grid(row=3, column=2, columnspan=2)
			MatchBut = Button(FFinal, text = 'Position on sequence', bg='yellow')
			MatchBut.config(command=sh2seq)
			MatchBut.grid()
			MatchBut.config(state=DISABLED)	

			Label(WinFragm, text=' ').grid(row=4)
			FOutND = Frame(WinFragm, relief=RIDGE, bd=2)
			FOutND.grid(row=5, column=0, columnspan=2)
			Label(FOutND,width=10, text='H-shift(pt)').grid(row=0, column=0, sticky='nswe')
			self.delataH = StringVar()	
			entDH = Entry(FOutND, width=10, bg='white', relief=RIDGE, textvariable = self.delataH)
			entDH.grid(row=0, column=1)
			entDH.insert(END,'0')

			FBut = Frame(WinFragm)
			FBut.grid(row=5, column=2, columnspan=2)
			SaveRes = Button(FBut, text='Peak pick ALL shapes', bg='white', command=SaveBBA)
			SaveRes.grid()
			SaveRes.config(state=DISABLED)	
                 
	def loadPkList(self, event=None):			
		[self.Lindshp, self.Ldshp, self.offset, self.NucType, self.SWppm, self.O1ppm, self.Size] = LoadComp()
		self.tpf = self.NucType[1::]
		
		if not len(self.Ldshp):
			tkMessageBox.showerror(title=None, message='Problem reading shapes!')
			self.TL.destroy()
		else:			
			# Shabba OUTPUT directory-> fixed
                	self.dirout = 'ShabbaOutput' + os.sep
			if os.path.exists(self.dirout): pass
                	else: os.mkdir(self.dirout)
			# Opening a log file for shabba
			self.flog = self.dirout+'shabba.log'
			f = open(self.flog,'w')
			f.write(time.ctime(time.time())+' The analysis of shapes has started.\n')				
			f.close()
								
			[x.config(state=NORMAL) for x in self.param]
			tkMessageBox.showinfo(title=None, message='Please CHECK the corresponding shape NUMBER (0..n)!')
			self.ButGly.config(state=NORMAL)
			self.ButNs.config(state=NORMAL)
		



if __name__ == "__main__":
	import pdb
	root = Tk()        
        app = comp2bba(root)
        root.withdraw()
	root.mainloop()
