#==Function for calculating rmsd
# f-> list with shifts, s-> sequence segment, pf-> penalty factor, k->indicator for Ca/Cb
def calcRMSD(f,s, pf, k):
	nf = len([x for x in f if x!=0])
	rf, rs, c = len(f), len(s), 0
        f2s, pf2s = [], []
        # chain is shorter then a sequence segment
        while (rf+c) <= rs:
                sumd = 0
                for j in range(rf):
                        # only elements that are !=0 are used
                        if f[j]:
                                if all([s[j+c]!=0,s[j+c]!=1]): sumd = sumd + (s[j+c]-f[j])**2
                                # applying penalty for Gly in Cb sequence
                                elif s[j+c]==0: sumd = sumd + pf**2
                                # applying penalty for Pro
                                else: 
                                        # Pro in the bigining of a fragment; no panalty; Cb(Pro)=31.90; Ca(Pro)=63.30
                                        if j==0: 
                                                if k==0: sumd = sumd + (31.90-f[j])**2	# Cb
                                                else: sumd = sumd + (63.30-f[j])**2	# Ca
                                        else:  sumd = sumd + pf**2

                        elif s[j+c]: sumd = sumd + pf**2
                # current fragment information: f2s->rmsd, pf2s-> position in f2s
                f2s.append(sqrt(sumd/nf))
                pf2s.append(c) # the same for Cb or Ca
                c=c+1
		
	return f2s,pf2s

#==Function for positioning chains of Ca and/or Cb shifts on a protein sequence
# Arguments: pf-> penalty factor
# compCb, compCa-> Cbeta, Calpha chemical shifts resp.
# ListChains-> List of component chains
def comp2seq(pf, ListChains, compCb,compCa, dirout): 	 
	# Initializing the output: list of [component, residue]
	BBAout = []
	# !Assuming both Ca and Cb are not empty
	if all([len(compCb),len(compCa)]):
		filein = tkFileDialog.askopenfile(title='Select FILE with protein sequence')
		if filein!=None:	
                        buf = filein.readlines()
                        filein.close()
                        seq = []
                        for i in range(len(buf)):
                                if len(buf[i].split()):	[seq.append(x) for x in buf[i].split()]
			# If the sequence consists of one-letter/residue with no separation
			if any([len(x) for x in seq if len(x)>3]): 
				temp = seq[0]
				if len(seq)>1:
					for s in seq[1::]: temp = temp+s
				seq = []
				for i in temp: seq.append(i)
			Nres = range(1,len(seq)+1)
                        print "The loaded sequence has",len(seq),"amino acids."
         		
                        #==Function for appending chem. shift values==
                        # Ca, dCa-> shift and stand. dev for Calpha; similar for Cbeta
			#  nHa, nHb-> number of Ha and Hb nuclei
                        def CSappend(Ca, dCa, Cb, dCb, nHa, nHb):
                                Ca_stat.append(Ca)
                                deltaCa.append(dCa)
                                Cb_stat.append(Cb)
                                deltaCb.append(dCb)
				NP_Ha.append(nHa)
				NP_Hb.append(nHb)
                                return 
                        # Statistical values from: http://www.bmrb.wisc.edu/ref_info/statsel.htm (Last updated: 11-22-2008)
                        Ca_stat, deltaCa, Cb_stat, deltaCb = [], [], [], []
			# Number of peaks expected for Ha, Hb for a residue
			NP_Ha, NP_Hb = [],[]
                        for aa in seq:
                                if len([x for x in ['Ala', 'ala', 'A', 'a', 'ALA'] if x==aa]):CSappend(53.17, 1.99, 18.97, 1.83, 1 ,1)
                                elif len([x for x in ['Arg','arg', 'R', 'r', 'ARG'] if x==aa]):CSappend(56.79, 2.33, 30.67, 1.86, 1, 2) 
                                elif len([x for x in ['Asn','asn', 'N', 'n', 'ASN'] if x==aa]):CSappend(53.54, 1.91, 38.67, 1.71, 1, 2)
                                elif len([x for x in ['Asp','asp', 'D', 'd', 'ASP'] if x==aa]):CSappend(54.67, 2.06, 40.86, 1.65, 1, 2)
                                elif len([x for x in ['Cys','cys', 'C', 'c', 'CYS'] if x==aa]):CSappend(58.06, 3.40, 33.06, 6.36, 1, 2)
                                elif len([x for x in ['Glu','glu', 'E', 'e', 'GLU'] if x==aa]):CSappend(57.36, 2.11, 29.99, 1.75, 1, 2)
                                elif len([x for x in ['Gln','gln', 'Q', 'q', 'GLN'] if x==aa]):CSappend(56.59, 2.15, 29.16, 1.85, 1, 2)
                                elif len([x for x in ['Gly','gly', 'G', 'g', 'GLY'] if x==aa]):CSappend(45.35, 1.30, 0, 0, 2, 0)
                                elif len([x for x in ['His','his', 'H', 'h', 'HIS'] if x==aa]):CSappend(56.46, 2.39, 30.21, 2.10, 1, 2)
                                elif len([x for x in ['Ile','ile', 'I', 'i', 'ILE'] if x==aa]):CSappend(61.61, 2.71, 38.60, 2.05, 1, 1)
                                elif len([x for x in ['Leu','leu', 'L', 'l', 'LEU'] if x==aa]):CSappend(55.65, 2.15, 42.29, 1.89, 1, 2)
                                elif len([x for x in ['Lys','lys', 'K', 'k', 'LYS'] if x==aa]):CSappend(56.98, 2.22, 32.76, 1.82, 1, 2)
                                elif len([x for x in ['Met','met', 'M', 'm', 'MET'] if x==aa]):CSappend(56.15, 2.25, 32.99, 2.25, 1, 2)
                                elif len([x for x in ['Phe','phe', 'F', 'f', 'PHE'] if x==aa]):CSappend(58.15, 2.62, 39.90, 2.08, 1, 2)
                                elif len([x for x in ['Pro','pro', 'P', 'p', 'PRO'] if x==aa]):CSappend(63.32, 1.57, 31.83, 1.22, 1, 2)
                                elif len([x for x in ['Ser','ser', 'S', 's', 'SER'] if x==aa]):CSappend(58.72, 2.12, 63.80, 1.50, 1, 2)
                                elif len([x for x in ['Thr','thr', 'T', 't', 'THR'] if x==aa]):CSappend(62.21, 2.65, 69.69, 1.79, 1, 1)
                                elif len([x for x in ['Trp','trp', 'W', 'w', 'TRP'] if x==aa]):CSappend(57.67, 2.58, 29.98, 2.02, 1, 2)
                                elif len([x for x in ['Tyr','tyr', 'Y', 'y', 'TYR'] if x==aa]):CSappend(58.12, 2.57, 39.30, 2.17, 1, 2)
                                elif len([x for x in ['Val','val', 'V', 'v', 'VAL'] if x==aa]):CSappend(62.46, 2.90, 32.72, 1.83, 1, 1)

                                else: print "!Error: The amino acid", aa, "has unknown format!"		
                        listCS = [Cb_stat, Ca_stat]
                        listDCS = [deltaCb, deltaCa]
                        posPro = [x for x in range(len(seq)) if seq[x] in ['Pro','pro', 'p', 'P', 'PRO']]
			
                        #==calculating RMSD for each chain iteratively starting from the longest one with max length                                            
                        rmsdCS= [[] for x in range(len(ListChains))]	# rmsd
                        prmsdCS= [[] for x in range(len(ListChains))]	# rmsd positon on the sequence
                        lngth = max([len(x) for x in compCa],[len(x) for x in compCb])	# if Ca/Cb missing
			# Order the input chains (start from the longest)
			# !Assuming the Ca and Cb are not empty
			copyCa = list(compCa)
			ScompCa, Sindx =[], []
			for x in sorted(lngth,reverse=True):
				for elem in copyCa: 
					if len(elem)==x: 
						ScompCa.append(elem)
						# assuming a chain appears only once
						Sindx.append([y for y in range(len(compCa)) if compCa[y]==elem][0])
						copyCa.remove(elem)						
						break
			ScompCb = [compCb[x] for x in Sindx]
			SLComp = [ListChains[x] for x in Sindx]
                        cutln = [len(x) for x in ScompCa]
                        var = cutln[0]
                        pcur_s = range(len(seq))	# positions for the current sequence
                        Lpcur_s = [pcur_s]		# start with the entire sequence
			IndPlot = []
			
			fl = open(dirout+'rmsd.log', 'w')	
                        while var:			
                                loc = [x for x in range(len(ScompCa)) if len(ScompCa[x])==var]
                                # if there is more then one chain with the same size				
                                for p in loc:			
                                        chain = [ScompCb[p], ScompCa[p]]	# if both Ca and Cb shifts are available                                       				
					fl.write('\nChain No '+ str(Sindx[p]+1)+' of length '+str(len(ListChains[Sindx[p]]))+'\n')
                                        for n, count in enumerate(Lpcur_s):	# for each segment of the sequence						
                                                if not n:	#initial set-up
                                                        rmsdCS[p] = [[] for x in range(len(Lpcur_s))] # for each chain of components
                                                        prmsdCS[p] = [[] for x in range(len(Lpcur_s))]
                                                
                                                for k in range(len(chain)):	# rmsd for Cb (k=0) OR Ca (k=1)								
                                                        f = chain[k]												
                                                        s = [listCS[k][x] for x in count]
							# output the rmsd for each chain along the complete sequence (only once per chain)
							if not n:
								if len([x for x in f if x!=0]):	
									report = calcRMSD(f,listCS[k], pf, k)
									if k: fl.write('rmsd for Ca\tposition(in sequence)\n')
									else: fl.write('rmsd for Cb\tposition(in sequence)\n')
									for x in range(len(report[0])): fl.write(str(report[0][x])+'\t'+str(report[1][x])+'\t'+str(Nres[report[1][x]])+seq[report[1][x]]+'\n')
									fl.write('Minimum of: '+str(min(report[0]))+' at position: '+str(report[1][report[0].index(min(report[0]))])+'\n')

                                                        # considered only nonzero elements in 'f' (zero elements are counted as Gly or missing Cb/Ca)			
                                                        if len([x for x in f if x!=0]):							
                                                                result = calcRMSD(f,s, pf, k)	
                                                                # p-> fragment No, n-> segment No, if rmsd empty							
                                                                if not len(rmsdCS[p][n]): rmsdCS[p][n], prmsdCS[p][n]= result[0], [min(count)+x for x in result[1]]		
                                                                else: rmsdCS[p][n]=[rmsdCS[p][n][x]+result[0][x] for x in result[1]]	# k=1 (Ca)					
					
                                        # reduce the sequence after chain(s) with size 'var' is(are) located (min rmsd)			
                                        MultRMSD = [x for x in range(len(rmsdCS[p])) if len(rmsdCS[p][x])]
					# Only the location with the minimum of all possibile ones is taken 
                                        if len(MultRMSD):
						Lmin = min([min(x) for x in rmsdCS[p]])
						excl = [x for x in range(len(rmsdCS[p])) if Lmin not in rmsdCS[p][x]]						 
						if len(excl): 
							[rmsdCS[p].remove(x) for x in [rmsdCS[p][x] for x in excl]]		
							[prmsdCS[p].remove(x) for x in [prmsdCS[p][x] for x in excl]]
						minloc = [x for x in range(len(rmsdCS[p][0])) if rmsdCS[p][0][x]==Lmin]
						if len(minloc)>1: 
							print 'Error: more then one minimum for Chain No '+ str(Sindx[p]+1)
							rmsdCS[p], prmsdCS[p] = [],[]
						elif len(minloc)==1:
							# check if the minloc positions the 2nd element(Ist component) of chain over Pro (no component can be Pro)
							if seq[prmsdCS[p][0][minloc[0]]+1] not in ['Pro','pro', 'P', 'p', 'PRO']:					                         
                                        			pcur_s = [x for x in pcur_s if x not in range(prmsdCS[p][0][minloc[0]]+1, prmsdCS[p][0][minloc[0]]+var-1)] # indices for sequence
								IndPlot.append(p)
							else: 	rmsdCS[p], prmsdCS[p] = [],[]
                                
                                Lpcur_s = []	
                                if len(pcur_s):	# left sequence part that is not compared                                        
                                        s_part = [pcur_s[0]]
                                        for i in range(1,len(pcur_s)):
                                                if pcur_s[i]==pcur_s[i-1]+1:s_part.append(pcur_s[i])
                                                else: 
                                                        Lpcur_s.append(s_part)
                                                        s_part=[pcur_s[i]]
                                        if s_part not in Lpcur_s: Lpcur_s.append(s_part)				
                                	# next is a shorter chain
                                	cutln = [x for x in cutln if x!=var]
                                	if len(cutln): 
						var = max(cutln)						
						Lpcur_s = [segm for segm in Lpcur_s if len(segm)>=var]						 
						if not len(Lpcur_s): var = 0 # !Stop comparing: no segments >= longest from the left chains 							
                                	else: var = 0 # no chain left to match						
				else: var = 0 # there is no left part of the sequence to match					
                        fl.close()
			
			#==Select chains/chemical shifts to be plotted (only chains that were matched) 	       
                        listRest = [[ScompCb[x] for x in IndPlot], [ScompCa[x] for x in IndPlot], [rmsdCS[x] for x in IndPlot]]	
			LPos = [prmsdCS[x] for x in IndPlot]
			SLComp = [SLComp[x] for x in IndPlot] # selecting only chains that were matched with the sequence
                        # Positioning the chains according to rmsd minimum (assuming there is only one!)
                        posCb, posCa = [],[]
			
                        for i, ch in enumerate(listRest[-1]):
                                for k in range(len(ch)):
                                        start_ch = [x for x in range(len(ch[k])) if ch[k][x]==min(ch[k])]					
                                        if len(start_ch)==1: 						
                                        	posCb.append(range(LPos[i][k][start_ch[0]], LPos[i][k][start_ch[0]]+len(listRest[0][i])))
                                                posCa.append(range(LPos[i][k][start_ch[0]], LPos[i][k][start_ch[0]]+len(listRest[1][i])))
                                        
                        listPos = [posCb, posCa, LPos]	
			# Assignment: Only components in the selected chains will be assigned 						
			arg=SLComp	
			assign= []
			assignCaCb = [[] for x in range(3)]			
			for j, elem in enumerate(arg):
				LabelComp = [str(Nres[x])+seq[x] for x in posCa[j]]	# it will include i-1 components				
				for m, res in enumerate(LabelComp):
					if res not in assignCaCb[0]:
						assignCaCb[0].append(res)
						assignCaCb[1].append(listRest[1][j][m]) # Ca
						assignCaCb[2].append(listRest[0][j][m]) # Cb
				
				#starting from the end of a chain that coincides with the end of corresponding Ca/Cb list
				ind = len(LabelComp)
				for i in range(len(elem)-1,-1,-1):
					ind = ind-1			
					assign.append(['comp' + elem[i], LabelComp[ind], '\tMinRMSD: ', str(round(min(listRest[-1][j][0]),2))])
					
			#==Order the output component-to-residue						
			for i, res in enumerate(seq): [BBAout.append(assign[x]) for x in range(len(assign)) if assign[x][1]==str(Nres[i])+res] 
			
                        # plotting type of the chains: possible combination of color(6),symbols(19) and lines(4) 
                        ln = ['-','--','-.',':']
                        simb = ['o','^','v','<','>','s','+','x','D','d','1','2','3','4','h','H','p','|','_']
                        clr = ['b','g','r','c','m','y']
                        pltype = []
                        for i in range(len(ln)):
                                for j in range(len(simb)):
                                        for k in range(len(clr)): pltype.append(ln[i]+simb[j]+clr[k])
                                       
                        listTitle = ['rmsd (ppm)','Cb (ppm)', 'Ca (ppm)']
			IniSize = gcf().get_size_inches()
                        figure(num=1,figsize=(IniSize[0],int(IniSize[1]/2)*3))			

			lst,lines = [], []
			font= FontProperties(size='small')
                        # Plot statistics: 1->rmsd; 2->Cbeta; 3->Calpha
                        for n in range(3):			
                                pn = str(31)+str(n+1)
                                subplot(pn)
				subplots_adjust(right=0.7)
                                if not n: 
                                        for ch in range(len(listRest[-1])):
						var0= 'chain No'+str(Sindx[ch]+1)
						if len(listRest[-1][ch][0]):						
                                                        elm, elmpos = listRest[-1][ch][0],listPos[-1][ch][0] 								
                                                        semilogy(elmpos,elm, pltype[ch][-1], label='_nolegend_')							
							if len(elm)==1:	 minpos = [0]					
                                                        else: minpos = [x for x in range(len(elm)) if elm[x]<10]								
                                                        plot([elmpos[x] for x in minpos],[elm[x] for x in minpos],'s'+pltype[ch][-1], \
								label = var0+': min(rmsd)='+str(round(min(elm),2))+' at pos='+str(elmpos[elm.index(min(elm))]))
							if os.sep == '\\': legend(loc = (1.1,0), handlelength = 0.03,prop=font)
							else:	legend(loc = (1.1,0), handlelen = 0.03,prop=font)
                                                        ylim(0.1,60)															
                                else:	
                                        m = n-1		
                                        # only non-zero values are plotted					
                                        select = list([x for x in range(len(listCS[m])) if listCS[m][x]!=0])
					sel_black = [x for x in select if x not in posPro]
					sel_white = [x for x in select if x in posPro]
                                        errorbar(sel_black,[listCS[m][x] for x in sel_black], yerr= [listDCS[m][x] for x in sel_black],fmt = 'ks', label='BioMagResBank')
					errorbar(sel_white,[listCS[m][x] for x in sel_white], yerr= [listDCS[m][x] for x in sel_white],fmt = 'ws', ecolor='k', label='BioMagResBank')
                                        # for each chain of components
                                        for ch in range(len(listRest[m])):
						var= 'chain No'+str(Sindx[ch]+1)+' of length '+str(len(ListChains[Sindx[ch]]))
                                                if len(listRest[m][ch]):	# in case of missing Ca or Cb shifts
                                                        indx = [x for x in range(len(listRest[m][ch])) if listRest[m][ch][x]!=0]		
                                                        plot([listPos[m][ch][x] for x in indx], [listRest[m][ch][x] for x in indx],pltype[ch], label=var)
							if m:
								if os.sep == '\\': legend(loc = (1.1,0.5), handlelength = 0.03,prop=font)
								else: legend(loc = (1.1,0.5), handlelen = 0.03,prop=font)
                                                ylim(ymin=10)					
						
                                if len(seq[0])>1: xticks(range(len(seq)),seq, rotation = 'vertical')
                                else: xticks(range(len(seq)),seq)
                                xlim((-1,len(seq)+1))			
                                ylabel(listTitle[n])
			
			figtext(0.5,0.95,'Backbone assignment', fontsize=15, horizontalalignment='center', verticalalignment='center')		
                        try:
				figure(1).show()
			except:
				show()
		else: print "No file with a sequence is selected!"
        else: print "Error: Missing chemical shifts!"
	
	# Output: BBAout: List of ['comp No', 'NSeq[N]', '\tMinRMSD: ', 'value']; assignCaCb: List of [List of'NSeq[N]', List of Ca, List of Cb]
	# Number of H nuclei(peaks) per residue from the sequence: NP_Ha, NP_Hb.
        return BBAout, assignCaCb, NP_Ha, NP_Hb, seq
        		
#==MAIN         
from pylab import *
from matplotlib.font_manager import fontManager, FontProperties
from Tkinter import *
import tkFileDialog, os


